from NuRadioMC.SignalGen.askaryan import get_time_trace, get_frequency_spectrum
from NuRadioMC.utilities import units, fft
import numpy as np
import matplotlib.pyplot as plt

E = 1e20 * units.eV
n_index = 1.78
domega = 0.05 * units.deg
theta = np.arccos(1. / n_index) + domega

dt = 0.1 * units.ns
n_samples = 2048
ff = np.fft.rfftfreq(n_samples, dt)
tt = np.arange(0, n_samples * dt, dt)
R = 10 * units.km

models = ['Alvarez2009', 'ARZ2019', 'Alvarez2000']
colours = ['blue', 'orange', 'red', 'black']
col_dict = {}
linestyle = {'Alvarez2009': '-', 'ARZ2019': '--', 'Alvarez2000': '-.'}
shower_type = 'EM'

Es = 10**np.linspace(15,17,3) * units.eV
domegas = np.linspace(-5,5,5) * units.deg
thetas = np.arccos(1./n_index) + domegas

for iE, E in enumerate(Es):
    col_dict[E] = colours[iE]

for model in models:

    for E in Es:

        spectrum = get_frequency_spectrum(E, theta, n_samples, dt, shower_type, n_index, R, model)
        spectrum *= R 
        freqs = np.fft.rfftfreq(n_samples, dt)

        if (model=='Alvarez2009'):
            label = str(E/units.EeV)+' EeV'
        else:
            label = ''

        #plt.plot(tt, trace, color=col_dict[E], label=label, linestyle=linestyle[model])
        plt.loglog(freqs, np.abs(spectrum) / units.V * units.MHz, color=col_dict[E], label=label, linestyle=linestyle[model])

#plt.xlabel('Time [ns]')
plt.xlabel('Frequency [GHz]')
plt.ylabel('Electric field [V/MHz]')
plt.legend()
plt.show()



import matplotlib.pyplot as plt
import numpy as np
from NuRadioReco.utilities import units
from NuRadioMC.utilities import fluxes
import os

energyBinsPerDecade = 2.
plotUnitsEnergy = units.GeV
plotUnitsFlux = units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1

# Unless you would like to work with the layout or the models/data from other experiments,
# you don't need to change anything below here
# --------------------------------------------------------------------
# Other planned experiments

# GRAND white paper,
# numerical values, Bustamante
GRAND_energy = np.array(([48192296.5, 67644231.1, 94947581.6, 133271428.0, 187063990.0, 262568931.0, 368550053.0, 517308507.0, 726110577.0, 1019191760.0, 1430569790.0, 2007992980.0, 2818482440.0, 3956111070.0, 5552922590.0, 7794257720.0, 10940266600.0, 15356104100.0, 21554313200.0, 30254315500.0, 42465913900.0,
                          59606499400.0, 83665567300.0, 117435636000.0, 164836371000.0, 231369543000.0, 324757606000.0, 455840043000.0, 639831498000.0, 898087721000.0, 1260584320000.0, 1769396010000.0, 2483580190000.0, 3486031680000.0, 4893104280000.0, 6868115880000.0, 9640304610000.0, 13531436400000.0, 18993151900000.0, 26659388600000.0]))

GRAND_energy *= units.GeV

GRAND_10k = np.array(([8.41513361e-08, 7.38147706e-08, 5.69225180e-08, 3.46647934e-08,
                       1.95651137e-08, 1.40651565e-08, 1.25782087e-08, 1.24621707e-08,
                       1.31123151e-08, 1.45812119e-08, 1.65528260e-08, 1.91930521e-08,
                       2.31554429e-08, 2.87477813e-08, 3.55164030e-08, 4.42563884e-08,
                       5.63965197e-08, 7.45183330e-08, 1.01159657e-07, 1.39040439e-07,
                       1.98526677e-07, 2.61742251e-07, 3.40870828e-07, 4.82745531e-07,
                       6.55876763e-07, 9.07706655e-07, 1.67125879e-06, 1.76142511e-05,
                       2.55022320e-04, 1.88371074e-03, 6.71431813e-03, 1.14286198e-02,
                       1.14294614e-02, 1.72447830e-02, 7.48579143e-02, 3.31883351e-01,
                       8.57786094e-01, 1.24824516e+00, 1.42294586e+00, 1.80135089e+00]))

GRAND_10k *= (units.GeV* units.cm**-2 * units.second**-1 * units.sr**-1)
#GRAND_10k /= 2 #halfdecade bins
GRAND_10k *= energyBinsPerDecade

GRAND_200k = np.array(([4.26219753e-09, 3.58147708e-09, 2.75670137e-09, 1.85254042e-09,
                        1.13825106e-09, 7.70141315e-10, 6.51758930e-10, 6.35878242e-10,
                        6.69261628e-10, 7.37439217e-10, 8.38784832e-10, 9.81688683e-10,
                        1.18493794e-09, 1.45699379e-09, 1.80867621e-09, 2.26948852e-09,
                        2.91952068e-09, 3.86790849e-09, 5.24530715e-09, 7.31211288e-09,
                        9.98848945e-09, 1.33523293e-08, 1.80893102e-08, 2.46582187e-08,
                        3.41054825e-08, 5.39140368e-08, 3.36553610e-07, 4.57179717e-06,
                        3.59391218e-05, 1.47550853e-04, 3.33777479e-04, 4.92873322e-04,
                        6.68381070e-04, 1.72553598e-03, 7.06643413e-03, 2.10754560e-02,
                        4.06319101e-02, 5.88162853e-02, 7.45423652e-02, 8.83700084e-02]))

GRAND_200k *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)

# RADAR proposed from https://arxiv.org/pdf/1710.02883.pdf

Radar = np.array(([
    (1.525e+01, 6.870e-09, 3.430e-07),
    (1.575e+01, 9.797e-10, 3.113e-08),
    (1.625e+01, 4.728e-09, 1.928e-07),
    (1.675e+01, 6.359e-09, 3.706e-07),
    (1.725e+01, 9.128e-09, 8.517e-07),
    (1.775e+01, 1.619e-08, 1.835e-06),
    (1.825e+01, 2.995e-08, 2.766e-06),
    (1.875e+01, 5.562e-08, 8.253e-06),
    (1.925e+01, 1.072e-07, 1.849e-05)]))

Radar[:, 0] = 10**Radar[:, 0] * units.eV
Radar[:, 1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
Radar[:, 2] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)

# --------------------------------------------------------------------
# Published data and limits

# IceCube
# log (E^2 * Phi [GeV cm^02 s^-1 sr^-1]) : log (E [Gev])
# Phys Rev D 98 062003 (2018)
# Numbers private correspondence Shigeru Yoshida
ice_cube_limit = np.array(([
    (6.199999125, -7.698484687),
    (6.299999496, -8.162876678),
    (6.400000617, -8.11395291),
    (6.500000321, -8.063634144),
    (6.599999814, -8.004841781),
    (6.699999798, -7.944960162),
    (6.799999763, -7.924197388),
    (6.899999872, -7.899315263),
    (7.299999496, -7.730561153),
    (7.699999798, -7.670680637),
    (8.100001583, -7.683379711),
    (8.500000321, -7.748746801),
    (8.899999872, -7.703060304),
    (9.299999496, -7.512907553),
    (9.699999798, -7.370926525),
    (10.10000158, -7.134626026),
    (10.50000032, -6.926516638),
    (10.89999987, -6.576523031)
]))

ice_cube_limit[:, 0] = 10**ice_cube_limit[:, 0] * units.GeV
ice_cube_limit[:, 1] = 10**ice_cube_limit[:, 1] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_limit[:, 1] *= energyBinsPerDecade

# Fig. 2 from PoS ICRC2017 (2018) 981
# IceCube preliminary
# E (GeV); E^2 dN/dE (GeV cm^-2 s-1 sr-1); yerror down; yerror up
ice_cube_hese = np.array(([

    (6.526e+04,  2.248e-08,  9.96e-9,  1.123e-8),
    (1.409e+05,  2.692e-08,  5.91e-9,  7.56e-9),
    (3.041e+05,  7.631e-09,  3.746e-9, 4.61e-9),
    (6.644e+05,  2.022e-09,  7.03e-10, 0.),
    (1.434e+06,  5.205e-09,  3.183e-9,  4.57e-9),
    (3.096e+06,  4.347e-09,  3.142e-9,  5.428e-9),
    (6.684e+06,  1.544e-09,  5.37e-10, 0.),
    (1.46e+07,  4.063e-09,   1.353e-9, 0.),
    (3.153e+07,  6.093e-09,  2.03e-9,  0.),
    (6.806e+07,  1.046e-08,  3.641e-9, 0.)
]))



ice_cube_hese[:, 0] = ice_cube_hese[:, 0] * units.GeV
ice_cube_hese[:, 1] = ice_cube_hese[:, 1] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_hese[:, 1] *= 3
ice_cube_hese[:, 2] = ice_cube_hese[:, 2] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_hese[:, 2] *=  3
ice_cube_hese[:, 3] = ice_cube_hese[:, 3] * (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
ice_cube_hese[:, 3] *= 3

# Ice cube


def ice_cube_nu_fit(energy, slope=-2.13, offset=0.9):
    flux = 3 * offset * (energy / (100 * units.TeV))**slope * 1e-18 * \
        (units.GeV**-1 * units.cm**-2 * units.second**-1 * units.sr**-1)
    return flux


def get_ice_cube_mu_range():
    energy = np.arange(1e5, 5e6, 1e5) * units.GeV
    upper = np.maximum(ice_cube_nu_fit(energy, offset=0.9, slope=-2.), ice_cube_nu_fit(energy, offset=1.2, slope=-2.13))
    upper *= energy**2
    lower = np.minimum(ice_cube_nu_fit(energy, offset=0.9, slope=-2.26),
                       ice_cube_nu_fit(energy, offset=0.63, slope=-2.13))
    lower *= energy**2
    return energy, upper, lower


def get_ice_cube_hese_range():
    energy = np.arange(1e5, 5e6, 1e5) * units.GeV
    upper = np.maximum(ice_cube_nu_fit(energy, offset=2.46, slope=-2.63),
                       ice_cube_nu_fit(energy, offset=2.76, slope=-2.92))
    upper *= energy**2
    lower = np.minimum(ice_cube_nu_fit(energy, offset=2.46, slope=-3.25),
                       ice_cube_nu_fit(energy, offset=2.16, slope=-2.92))
    lower *= energy**2
    return energy, upper, lower


# ANITA I - III
# Phys. Rev. D 98, 022001 (2018)
anita_limit = np.array(([
    (9.94e17,	3.79e-14 * 9.94e17 / 1e9),
    (2.37e18,	2.15e-15 * 2.37e18 / 1e9),
    (5.19e18,	2.33e-16 * 5.19e18 / 1e9),
    (1.10e19,	3.64e-17 * 1.10e19 / 1e9),
    (3.55e19,	4.45e-18 * 3.55e19 / 1e9),
    (1.11e20,	9.22e-19 * 1.11e20 / 1e9),
    (4.18e20,	2.97e-19 * 4.18e20 / 1e9),
    (9.70e20,	1.62e-19 * 9.70e20 / 1e9)
]))

anita_limit[:, 0] *= units.eV
anita_limit[:, 1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
anita_limit[:, 1] /= 2
anita_limit[:, 1] *= energyBinsPerDecade


# Auger neutrino limit
# Auger 9 years, all flavour (x3)
auger_limit = np.array(([
    (16.7523809524, 4.462265901e-07),
    (17.2523809524, 1.103901153e-07),
    (17.7523809524, 6.487559078e-08),
    (18.2380952381, 7.739545498e-08),
    (18.7523809524, 1.387743075e-07),
    (19.2571428571, 3.083827665e-07),
    (19.7523809524, 7.467202523e-07),
    (20.2476190476, 1.998499395e-06)
]))
auger_limit[:, 0] = 10 ** auger_limit[:, 0] * units.eV
auger_limit[:, 1] *= (units.GeV * units.cm**-2 * units.second**-1 * units.sr**-1)
auger_limit[:, 1] /= 2  # half-decade binning
auger_limit[:, 1] *= energyBinsPerDecade

# get 10% proton flux
def get_proton_10(energy):
    vanVliet_reas = np.loadtxt(os.path.join(os.path.dirname(__file__), "ReasonableNeutrinos1.txt"))
    E = vanVliet_reas[0, :] * plotUnitsEnergy
    f = vanVliet_reas[1, :] * plotUnitsFlux / E**2
    from scipy.interpolate import interp1d
    getE = interp1d(E, f)
    return getE(energy)
    


def get_E2_limit_figure(show_ice_cube_EHE_limit=True,
                        show_ice_cube_HESE=True,
                        show_ice_cube_mu=True,
                        show_anita_I_III_limit=True,
                        show_auger_limit=True,
                        show_neutrino_best_fit=True,
                        show_neutrino_best_case=True,
                        show_neutrino_worst_case=True,
                        show_grand_10k=True,
                        show_grand_200k=True,
                        show_radar=False):

    # Limit E2 Plot
    # ---------------------------------------------------------------------------
    fig, ax = plt.subplots(1, 1, figsize=(7, 8))

    # Neutrino Models

    Heinze_band = np.loadtxt(os.path.join(os.path.dirname(__file__), "talys_neu_bands.out"))
    best_fit, = ax.plot(Heinze_band[:, 0], Heinze_band[:, 1] * Heinze_band[:, 0]**2, c='k',
                        label=r'Best fit UHECR ($\pm$ 3$\sigma$), Heinze et al.', linestyle='-.')

    Heinze_evo = np.loadtxt(os.path.join(os.path.dirname(__file__), "talys_neu_evolutions.out"))
    ax.fill_between(Heinze_evo[:, 0], Heinze_evo[:, 5] * Heinze_evo[:, 0] **
                    2, Heinze_evo[:, 6] * Heinze_evo[:, 0]**2, color='0.8')

    vanVliet_max_1 = np.loadtxt(os.path.join(os.path.dirname(__file__), "MaxNeutrinos1.txt"))
    vanVliet_max_2 = np.loadtxt(os.path.join(os.path.dirname(__file__), "MaxNeutrinos2.txt"))
    vanVliet_reas = np.loadtxt(os.path.join(os.path.dirname(__file__), "ReasonableNeutrinos1.txt"))

    vanVliet_max = np.maximum(vanVliet_max_1[1, :], vanVliet_max_2[1, :])

    prot10, = ax.plot(vanVliet_reas[0, :], vanVliet_reas[1, :],
                      label=r'10% protons in UHECRs, van Vliet et al.', linestyle=':', color='darkmagenta')

    prot = ax.fill_between(vanVliet_max_1[0,:],vanVliet_max,vanVliet_reas[1,:]/10,color='thistle',alpha=0.5,label=r'not excluded from UHECRs')

    first_legend = plt.legend(handles=[best_fit, prot, prot10], loc=4)

    plt.gca().add_artist(first_legend)
    #-----------------------------------------------------------------------

    if show_grand_10k:
        ax.plot(GRAND_energy / plotUnitsEnergy, GRAND_10k / plotUnitsFlux, linestyle="--", color='saddlebrown')
        ax.annotate('GRAND 10k',
                    xy=(0.3e10, 1.1e-7 /2 * energyBinsPerDecade), xycoords='data',
                    horizontalalignment='left', color='saddlebrown', rotation=50)

    if show_grand_200k:
        ax.plot(GRAND_energy / plotUnitsEnergy, GRAND_200k / plotUnitsFlux, linestyle="--", color='saddlebrown')
        ax.annotate('GRAND 200k',
                    xy=(1e10, 6e-9 /2 * energyBinsPerDecade), xycoords='data',
                    horizontalalignment='left', color='saddlebrown', rotation=50)
    if show_radar:
        ax.fill_between(Radar[:, 0] / plotUnitsEnergy, Radar[:, 1] / plotUnitsFlux,
                        Radar[:, 2] / plotUnitsFlux, facecolor='None', hatch='x', edgecolor='0.8')
        ax.annotate('Radar',
                    xy=(1e9, 4.5e-8), xycoords='data',
                    horizontalalignment='left', color='0.7', rotation=45)

    if show_ice_cube_EHE_limit:
        ax.plot(ice_cube_limit[:, 0] / plotUnitsEnergy, ice_cube_limit[:, 1] / plotUnitsFlux, color='dodgerblue')
        ax.annotate('IceCube',
                    xy=(3e5, 1e-7), xycoords='data',
                    horizontalalignment='left', verticalalignment='center', color='dodgerblue', rotation=0)

    if show_ice_cube_HESE:
        # data points
        uplimit = np.copy(ice_cube_hese[:, 3])
        uplimit[np.where(ice_cube_hese[:, 3] == 0)] = 1
        uplimit[np.where(ice_cube_hese[:, 3] != 0.)] = 0

        ax.errorbar(ice_cube_hese[:, 0] / plotUnitsEnergy, ice_cube_hese[:, 1] / plotUnitsFlux, yerr=ice_cube_hese[:,2:].T / plotUnitsFlux,
                    uplims=uplimit, color='dodgerblue', marker='o', ecolor='dodgerblue', linestyle='None')

        # hese fit
        ice_cube_hese_range = get_ice_cube_hese_range()
        ax.fill_between(ice_cube_hese_range[0] / plotUnitsEnergy, ice_cube_hese_range[1] / plotUnitsFlux,
                        ice_cube_hese_range[2] / plotUnitsFlux, hatch='//', edgecolor='dodgerblue', facecolor='azure')
        plt.plot(ice_cube_hese_range[0] / plotUnitsEnergy, ice_cube_nu_fit(ice_cube_hese_range[0],
                                                                           offset=2.46, slope=-2.92) * ice_cube_hese_range[0]**2 / plotUnitsFlux, color='dodgerblue')

    if show_ice_cube_mu:
        # mu fit
        ice_cube_mu_range = get_ice_cube_mu_range()
        ax.fill_between(ice_cube_mu_range[0] / plotUnitsEnergy, ice_cube_mu_range[1] / plotUnitsFlux,
                        ice_cube_mu_range[2] / plotUnitsFlux, hatch='\\', edgecolor='dodgerblue', facecolor='azure')
        plt.plot(ice_cube_mu_range[0] / plotUnitsEnergy,
                 ice_cube_nu_fit(ice_cube_mu_range[0], offset=0.9, slope=-2.13) * ice_cube_mu_range[0]**2 / plotUnitsFlux,
                 color='dodgerblue')

    if show_anita_I_III_limit:
        ax.plot(anita_limit[:, 0] / plotUnitsEnergy, anita_limit[:, 1] / plotUnitsFlux, color='darkorange')
        ax.annotate('ANITA I - III',
                    xy=(5e9, 0.5e-6 /2 * energyBinsPerDecade), xycoords='data',
                    horizontalalignment='right', color='darkorange')

    if show_auger_limit:
        ax.plot(auger_limit[:, 0] / plotUnitsEnergy, auger_limit[:, 1] / plotUnitsFlux, color='forestgreen')
        ax.annotate('Auger',
                    xy=(1.2e8, 2.1e-7 /2 * energyBinsPerDecade), xycoords='data',
                    horizontalalignment='left', color='forestgreen', rotation=0)

    ax.set_yscale('log')
    ax.set_xscale('log')

    ax.set_xlabel(r'Neutrino Energy [GeV]')
    ax.set_ylabel(r'$E^2\Phi$ [GeV cm$^{-2}$ s$^{-1}$ sr$^{-1}$]')

    ax.set_ylim(1e-11, 2e-6)
    ax.set_xlim(1e5, 1e11)

    plt.tight_layout()
    return fig, ax


def add_limit(ax, limit_labels, E, Veff, n_stations, label, livetime=3*units.year, fmt='-', color='C0'):
    E = np.array(E)
    Veff = np.array(Veff)
    limit = fluxes.get_limit_e2_flux(energy = E,
                                     veff = Veff,
                                     livetime = livetime,
                                     signalEff = n_stations,
                                     energyBinsPerDecade=energyBinsPerDecade,
                                     upperLimOnEvents=2.44,
                                     nuCrsScn='ctw')

    _plt, = ax.plot(E/plotUnitsEnergy,limit/ plotUnitsFlux, fmt, color=color,
                    label="{2}: {0} stations, {1} years".format(n_stations,int(livetime/units.year),label),
                    linewidth=3)
    limit_labels.append(_plt)
    return limit_labels

def add_limit_band(ax, limit_labels, E, Veff1, Veff2, n_stations, label, livetime=3*units.year, color='C0'):
    E = np.array(E)
    Veff1 = np.array(Veff1)
    Veff2 = np.array(Veff2)
    limit1 = fluxes.get_limit_e2_flux(energy = E,
                                     veff = Veff1,
                                     livetime = livetime,
                                     signalEff = n_stations,
                                     energyBinsPerDecade=energyBinsPerDecade,
                                     upperLimOnEvents=2.44,
                                     nuCrsScn='ctw')
    limit2 = fluxes.get_limit_e2_flux(energy = E,
                                      veff = Veff2,
                                     livetime = livetime,
                                     signalEff = n_stations,
                                     energyBinsPerDecade=energyBinsPerDecade,
                                     upperLimOnEvents=2.44,
                                     nuCrsScn='ctw')

    ax.fill_between(E/plotUnitsEnergy,limit1/ plotUnitsFlux,limit2/ plotUnitsFlux,
                            color=color, alpha=0.5, 
                            label="{2}: {0} stations, {1} years".format(n_stations,int(livetime/units.year),label))
#     limit_labels.append(_plt)
    return limit_labels

if __name__=="__main__":
    strawman_veff_pa = np.array(( [1.00000000e+16, 3.16227766e+16, 1.00000000e+17, 3.16227766e+17, 1.00000000e+18, 3.16227766e+18, 1.00000000e+19, 3.16227766e+19],
                              [1.82805666e+07, 1.34497197e+08, 6.32044851e+08, 2.20387046e+09, 4.86050340e+09, 8.18585201e+09, 1.25636305e+10, 1.83360237e+10])).T

    strawman_veff_pa[:,0] *= units.eV
    strawman_veff_pa[:,1] *= units.m**3 * units.sr
    
    strawman_pa_label = 'Strawman + PA@15m@2s'
    fig, ax = get_E2_limit_figure()
    labels =[]
    labels = add_limit(ax, labels, strawman_veff_pa[:,0],strawman_veff_pa[:,1], 270, strawman_pa_label)
    plt.legend(handles=labels, loc=2)
    plt.show()



import numpy as np
import h5py
from NuRadioReco.detector import detector
from NuRadioMC.utilities import units
from scipy import constants
from matplotlib import pyplot as plt
import glob
from radiotools import plthelpers as php
import sys
import os

Tnoise = 300
Vrms = (Tnoise * 50 * constants.k * 500 * units.MHz / units.Hz) ** 0.5

counter = -1
fig, ax = plt.subplots(1, 1, figsize=(7,7))
for iF, filename in enumerate(sorted(glob.glob(os.path.join(sys.argv[1], "*.hdf5")))):

    fin = h5py.File(filename)
    print(np.log10(fin.attrs['Emin']))
    if(np.log10(fin.attrs['Emin']) not in [17.0, 18.0, 19.0, 20.0]):
        continue
    counter += 1
    
    with open('det.json', 'w') as fout:
        fout.write(fin.attrs['detector'])
        fout.close()
    det = detector.Detector(json_filename="det.json")
    max_amps_env = np.array(fin['station_101']['maximum_amplitudes_envelope'])
    weights = fin['weights']
    

    n_stations = det.get_number_of_channels(101) / 4  # (we had 4 antennas per station)
    xs = np.zeros(n_stations)
    ys = np.zeros(n_stations)

    triggered_near_surface = np.any(max_amps_env[:, 0:3] > (3 * Vrms), axis=1)  # triggered any LPDA or dipole of the center station
    triggered_near_deep = max_amps_env[:, 3] > (3 * Vrms) # triggered deep dipole
    triggered_surface = np.zeros((max_amps_env.shape[0], n_stations)) # create empy array of shape (n events, n stations)
    triggered_deep = np.zeros((max_amps_env.shape[0], n_stations)) # create empy array of shape (n events, n stations)
    # loop through all stations with different distances 
    for i in range(n_stations):
        # select the 2 LPDA + 1 dipole channel and check if they fulfill the trigger condition
        triggered_surface[:, i] = np.any(max_amps_env[:, i * 4:(i * 4 + 3)] > (3 * Vrms), axis=1)
        triggered_deep[:, i] = max_amps_env[:, i * 4 + 3] > (3 * Vrms)
        # get their position
        xs[i] = np.abs(det.get_relative_position(101, i * 4)[0])
        ys[i] = np.abs(det.get_relative_position(101, i * 4)[1])

    # loop through all simulated distances
    coincidence_fractions_surface = np.zeros(len(np.unique(xs)))
    coincidence_fractions_deep = np.zeros(len(np.unique(xs)))
    for i, x in enumerate(np.unique(xs)):
        mask = ((np.abs(xs) == x) & (np.abs(ys) == x))  \
            | ((np.abs(xs) == x) & (ys == 0)) \
            | ((np.abs(ys) == x) & (xs == 0))
        # calculate coincidence fraction
        coincidence_fractions_surface[i] = 1. * np.sum(weights[np.any(triggered_surface[:, mask], axis=1) & triggered_near_surface]) / np.sum(weights[triggered_near_surface])
        coincidence_fractions_deep[i] = 1. * np.sum(weights[np.any(triggered_deep[:, mask], axis=1) & triggered_near_deep]) / np.sum(weights[triggered_near_deep])

    ax.plot(np.unique(xs), coincidence_fractions_surface, php.get_marker2(counter)+'-', label="E = {:.2g}eV".format(fin.attrs['Emin']))
    ax.plot(np.unique(xs), coincidence_fractions_deep, php.get_marker2(counter)+'--') #, label="E = {:.2g}, 50m deep".format(fin.attrs['Emin']))
ax.set_xlabel("distance [m]")
ax.set_ylabel("coincidence fraction")
ax.semilogy(True)
ax.set_aspect('equal', 'box')
ax.set_xlim(0, 3000)
ax.set_ylim(0.5e-2, 1.1)
ax.legend(loc='lower left', fontsize='medium', numpoints=1)
fig.tight_layout()
fig.savefig("coincidences.png")
fig.savefig("coincidences.pdf", bbox='tight')
plt.show()



import numpy as np
import h5py

from NuRadioMC.EvtGen import generator

datasets = {}
datasets['event_ids'] = np.arange(100, dtype=np.int)
datasets['zeniths'] = np.random.uniform(size=100)

additional_datasets = {}
additional_datasets['event_ids'] = np.array([0, 1, 10, 10, 10])
additional_datasets['zeniths'] = np.random.uniform(size=5)

generator.write_events_to_hdf5("T01.hdf5", data_sets=datasets, attributes={}, additional_interactions=additional_datasets)
generator.write_events_to_hdf5("T01_split10.hdf5", data_sets=datasets, attributes={}, additional_interactions=additional_datasets, n_events_per_file=10)

generator.split_hdf5_input_file("T01.hdf5", "T01_split10_later.hdf5", number_of_events_per_file=10)


#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
import sys
import h5py
import numpy as np
from numpy import testing
import argparse
from NuRadioReco.utilities import units
import logging

error = 0

file1 = sys.argv[1]
file2 = sys.argv[2]
print("Testing the files {} and {} for equality".format(file1, file2))

fin1 = h5py.File(file1, 'r')
fin2 = h5py.File(file2, 'r')

attributes = [u'trigger_names',
 u'Tnoise',
 u'Vrms',
 u'dt',
 u'bandwidth',
 u'n_samples',
 u'thetamin',
 u'zmax',
 u'zmin',
 u'thetamax',
 u'header',
 u'fiducial_zmax',
 u'fiducial_zmin',
 u'flavors',
 u'rmin',
 u'total_number_of_events',
 u'deposited',
 u'phimax',
 u'phimin',
 u'Emin',
 u'rmax',
 u'fiducial_rmax',
 u'Emax',
 u'fiducial_rmin',
 u'n_events']
for key in attributes:
    try:
        testing.assert_equal(fin1.attrs[key], fin2.attrs[key])
    except AssertionError as e:
        print("\n attribute {} not equal".format(key))
        print(e)


keys = [u'azimuths',
 u'energies',
 u'event_ids',
 u'flavors',
 u'inelasticity',
 u'interaction_type',
 u'multiple_triggers',
 u'n_interaction',
 u'triggered',
 u'weights',
 u'xx',
 u'yy',
 u'zeniths',
 u'zz']
for key in keys:
    try:
        testing.assert_equal(np.array(fin1[key]), np.array(fin2[key]))
    except AssertionError as e:
        print("\narray {} not equal".format(key))
        print(e)
        error = -1


keys2 = [u'SNRs',
 u'maximum_amplitudes',
 u'maximum_amplitudes_envelope',
 u'multiple_triggers',
 u'ray_tracing_solution_type',
 u'triggered']
for key in keys2:
    try:
#         testing.assert_allclose(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]), rtol=1e-9)
        testing.assert_almost_equal(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]))
    except AssertionError as e:
        print("\narray {} of group station_101 not equal".format(key))
        print(e)
        error = -1
        
keys2 = [
    u'travel_distances',
    u'ray_tracing_C1',]
for key in keys2:
    try:
#         testing.assert_allclose(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]), rtol=1e-9, atol=1*units.mm)
        testing.assert_almost_equal(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]))
    except AssertionError as e:
        print("\narray {} of group station_101 not equal".format(key))
        print(e)
        error = -1
        
keys2 = [
    u'travel_times']
for key in keys2:
    try:
#         testing.assert_allclose(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]), rtol=1e-9, atol=10*units.ps)
        testing.assert_almost_equal(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]))
    except AssertionError as e:
        print("\narray {} of group station_101 not equal".format(key))
        print(e)
        error = -1

keys2 = [
 u'polarization',
 u'ray_tracing_C0',
 u'launch_vectors',
 u'receive_vectors',
 ]
for key in keys2:
    try:
#         testing.assert_allclose(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]), rtol=1e-9, atol=1e-6)
        testing.assert_almost_equal(np.array(fin1['station_101'][key]), np.array(fin2['station_101'][key]))
    except AssertionError as e:
        print("\narray {} of group station_101 not equal".format(key))
        print(e)
        error = -1

    

if(error == -1):
    sys.exit(error)
else:
    print("The two files are identical.")






from __future__ import absolute_import, division, print_function
import numpy as np
import argparse
from radiotools import helper as hp
from NuRadioMC.utilities import units
from io import BytesIO
import logging
logger = logging.getLogger("readARAEventList")

VERSION = 0.1


def read_ARA_eventlist(filename):
    ara_version = 0
    event_number = 0
    with open(filename, 'r') as fin:
        lines = fin.readlines()
        data = ""
        for i, line in enumerate(lines):
            if line.startswith("VERSION"):
                ara_version = float(line.split("=")[1])
            elif line.startswith("EVENT_NUM"):
                event_number = int(line.split("=")[1])
            else:
                data += "{}".format(line)
        if(ara_version != 0.1):
            print("file version is {}. version != 0.1 not supported".format(ara_version))
            import sys
            sys.exit(-1)

        data = np.genfromtxt(BytesIO(data), comments='//', skip_header=3,
                             dtype=[('eventId', int), ('nuflavorint', int),
                                    ('nu_nubar', int), ('pnu', float),
                                    ('currentint', float),
                                    ('posnu_r', float), ('posnu_theta', float),
                                    ('posnu_phi', float), ('nnu_theta', float),
                                    ('nnu_phi', float), ('elast_y', float)])
        # convert angles into NuRadioMC coordinate convention
        for i in range(len(data)):
            data[i][3] = 10**(data[i][3] + 18.) * units.eV
            data[i][4] = data[i][4] * units.m
            data[i][6] = hp.get_normalized_angle(0.5 * np.pi - data[i][6])  # convert theta angle into NuRadioMC coordinate convention
            data[i][8] = hp.get_normalized_angle(0.5 * np.pi - data[i][8])  # convert theta angle into NuRadioMC coordinate convention
        return data


def convert_to_hdf5(araeventlist_filename, hdf5_filename):
    import h5py
    data = read_ARA_eventlist(araeventlist_filename)
    print(data)
    fout = h5py.File(hdf5_filename, 'w')
    fout['eventlist'] = data
    fout.attrs['VERSION'] = VERSION
    print(np.array(fout['eventlist']))
    fout.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Parse ARA event list.')
    parser.add_argument('filename', type=str,
                        help='path to ARA event list')
    parser.add_argument('output_filename', type=str,
                        help='name of hdf5 output filename')
    args = parser.parse_args()
    convert_to_hdf5(args.filename, args.output_filename)



import numpy as np
from NuRadioMC.utilities import units, fft
from scipy import constants
from scipy.optimize import curve_fit
import logging
logger = logging.getLogger("HCRB2017")
logger.setLevel(logging.INFO)


"""
Implementation of J. Hanson and A. Conolly "Complex analysis of Askaryan radiation: A fully analytic treatment
including the LPM effect and Cascade Form Factor." Astropart. Phys. 91, 75-89 (2017) which is based on
Buniy, R. V., Ralston, J. P.  "Radio detection of high energy particles: Coherence versus multiple scales"
Physical Review D, 65 016003 (2001)

This module uses a Gaisser Hillas shower profile for hadronic showers and a Greisen profile for EM showers. When 
the LPM effect is activated (default) instead of a Greisen profile, a parameterization from [1] is used. Please not that
these two parameterization (with/without LPM) do not give consistent results at energies where the LPM effect is
negligible. Use the model prediction with caution.

Please also note that the timing is not properly implemented (see https://github.com/nu-radio/NuRadioMC/issues/19).  

"""
speed_of_light = constants.c * units.m / units.s

_strictLowFreqLimit = True

NORM = 1.0

ICE_DENSITY = 0.9167  * units.g / units.cm**3
ICE_RAD_LENGTH = 36.08 * units.g / units.cm**2


def get_time_trace(energy, theta, N, dt, is_em_shower, n_index, R, LPM=True, a=None):
    """
    returns the Askaryan pulse in the time domain

    Parameters
    ----------
    energy : float
        energy of the shower
    theta: float
        viewangle: angle between shower axis (neutrino direction) and the line
        of sight between interaction and detector
    N : int
        number of samples in the time domain
    dt: float
        time bin width, i.e. the inverse of the sampling rate
    is_em_shower: bool
        true if EM shower, false otherwise
    n: float
        index of refraction at interaction vertex
    R: float
        distance from vertex to observer
    LPM: bool (default True)
        enable/disable LPD effect
    a: float or None (default Nont)
        if variable set, the shower width is manually set to this value
    """
    
    freqs = np.fft.rfftfreq(N, dt)
    eR, eTheta = _get_E_omega(freqs, energy, R, theta, n_index, is_em_shower, LPM, a=a)
    traceR = np.fft.irfft(eR) / dt
    traceTheta = np.fft.irfft(eTheta) / dt
    return np.array([traceR, traceTheta, np.zeros_like(traceTheta)])


def get_frequency_spectrum(energy, theta, N, dt, is_em_shower, n, R, LPM=True, a=None):
    """
    returns the complex amplitudes of the frequency spectrum of the neutrino radio signal

    Parameters
    ----------
    energy : float
        energy of the shower
    theta: float
        viewangle: angle between shower axis (neutrino direction) and the line
        of sight between interaction and detector
    N : int
        number of samples in the time domain
    dt: float
        time bin width, i.e. the inverse of the sampling rate
    is_em_shower: bool
        true if EM shower, false otherwise
    n: float
        index of refraction at interaction vertex
    R: float
        distance from vertex to observer
    LPM: bool (default True)
        enable/disable LPD effect
    a: float or None (default Nont)
        if variable set, the shower width is manually set to this value
    """
    eR, eTheta, ePhi = get_time_trace(energy, theta, N, dt, is_em_shower, n, R, LPM, a=a)
    return np.array([fft.time2freq(eR), fft.time2freq(eTheta), fft.time2freq(ePhi)])


def _get_k(ff, n_index):
    return 2 * np.pi * ff / speed_of_light * n_index


def _get_eta(k, _askaryanDepthA, _askaryanR, _askaryanTheta):
    return k * _askaryanDepthA**2 / _askaryanR * np.sin(_askaryanTheta)**2


def _get_Iff(ff, n_index, _askaryanDepthA, _askaryanR, _askaryanTheta):
    COS_THETA_C = 1. / n_index
    k = _get_k(ff, n_index)
    eta = _get_eta(k, _askaryanDepthA, _askaryanR, _askaryanTheta)
    re_d = 1 - 3 * eta**2 * np.cos(_askaryanTheta) / np.sin(_askaryanTheta)**2 * \
        (np.cos(_askaryanTheta) - COS_THETA_C) / (1 + eta**2)
    im_d = -eta - 3 * eta**3 * np.cos(_askaryanTheta) / np.sin(_askaryanTheta)**2 * \
        (np.cos(_askaryanTheta) - COS_THETA_C) / (1 + eta**2)
    denom = re_d + 1j * im_d
    re_power = -0.5 * (k * _askaryanDepthA)**2 * (np.cos(_askaryanTheta) - COS_THETA_C)**2 / (1 + eta**2)
    im_power = -eta * 0.5 * (k * _askaryanDepthA)**2 * (np.cos(_askaryanTheta) - COS_THETA_C)**2 / (1 + eta**2)
    power = re_power + 1j * im_power
    return np.exp(power) / denom**0.5


def _get_E_omega(ff, E, R, theta, n_index, EM=True,
                LPM=True, use_form_factor=True,
                _rho0=1. / (np.sqrt(2.0 * np.pi) * 0.03 * units.m),
                a=None, fudge_LPM=False):
    """
    calculates the frequncy spectrum of an Askaryan pulse. Do not use this function directly,
    use get_frequency_spectrum() instead
    
    Parameters
    -----------
    ff: np.array of floats
        array of frequencies
    E: float
        shower energy
    R: float
        distance from vertex to observer
    theta: float
        viewing angle
    n_index: float
        index of refraction at the shower
    EM: bool (default True)
        switch between EM and had. showers
    LPM: bool (default True)
        enable/disable LPD effect
    use_form_factor: bool (default True)
        use form factor
    _rho0: float
        the value of rho0
    a: float or None (default Nont)
        if variable set, the shower width is manually set to this value
    fudge_LPM: bool (default False)
        if True, the shower width parameterization of LPM showers is rescaled to match
        the Greisen parameterization at energies below the E_LPM, i.e., at energies where the LPM effect is negligible
        
    Returns:
        eR, eTheta component of electric field in frequency domain
    
    """

    _Nmax, _askaryanDepthA = get_N_AskDepthA(E, EM, LPM, fudge_LPM=fudge_LPM)
    if(a is not None):
        _askaryanDepthA = a
    COS_THETA_C = 1. / n_index
    k = _get_k(ff, n_index)
    eta = _get_eta(k, _askaryanDepthA, R, theta)
    I_FF = _get_Iff(ff, n_index, _askaryanDepthA, R, theta)
    nu = speed_of_light * k / (2.0 * np.pi)
    logger.debug("a {}, nmax {}, R {}".format(_askaryanDepthA, _Nmax, R))
    norm = 2.52e-7 * 1e3 * _askaryanDepthA * _Nmax * nu / R / NORM  # the additional *1e3 comes from putting all the units in the constant of Eq. (10). The left side of the equation is in MHz, whereas the right side is in GHz
    # Kinematic factor, psi...checked JCH March 8th, 2016...fixed missing sin(theta)
    psi = np.sin(theta) * np.sin(k * R) + 1j * (-np.sin(theta) * np.cos(k * R))
    # radial component (imaginary part is zero)...checked JCH March 8th, 2016
    rComp_num = -(np.cos(theta) - COS_THETA_C) / np.sin(theta)
    rComp = I_FF * norm * psi * rComp_num
    # theta component (has real and imaginary parts)...checked JCH March 8th, 2016
    thetaComp_num = 1 + eta**2 / (1 + eta)**2 * COS_THETA_C / np.sin(theta)**2 * (np.cos(theta) - COS_THETA_C) + \
        1j * (-eta / (1 + eta)**2 * COS_THETA_C / np.sin(theta)**2 * (np.cos(theta) - COS_THETA_C))
    thetaComp = I_FF * norm * psi * thetaComp_num
    logger.debug("IFF[0] {:.2g}, norm {:.2g}, psi[0] {:.2g}, thetaComp_num {:.2g}".format(I_FF[1], norm[1], psi[1], thetaComp_num[1]))

    if use_form_factor:
        a = k / _rho0
        b = np.sin(theta) / (2.0 * np.pi)**0.5
        atten = (1 + a**2 * b**2)**-1.5
        rComp *= atten
        thetaComp *= atten

    return rComp, thetaComp

def gauss(x, A, mu, sigma):
    return A * np.exp(-(x-mu)**2/2/sigma**2)


def get_N_AskDepthA(E, EM=True, LPM=True, fudge_LPM=False):
    """
    calculates the Gaussian width (sigma) of the shower profile using the 
    Greisen profile for EM showers and the Gaisser-Hillas profile for HAD showers. 
    If the LPM flag is activated, for EM shower of the shower width of 10.1103/PhysRevD.82.074017 is used.
    
    Please note that the parameterization of the shower width for LPM showers is not compatible with the Greisen
    parameterization event at regimes where the LPM effect is negligible!!!
    
    Parameters
    ----------
    E: float
        the energy of the shower
    EM: bool (default True)
        switch between EM and had. showers
    LPM: bool (default True)
        enable/disable LPD effect
    fudge_LPM: bool (default False)
        if True, the shower width parameterization of LPM showers is rescaled to match
        the Greisen parameterization at energies below the E_LPM, i.e., at energies where the LPM effect is negligible
    """
    if EM:
        E_CRIT = 0.073 * units.GeV  # GeV
        max_x = 5000.0  # maximum number of radiation lengths
        dx = 0.01  # small enough bin in depth for our purposes.
        x_start = 0.01  # starting radiation length
        # Greissen EM shower profile from Energy E in GeV.

        x = np.arange(x_start, max_x, dx)
        a = 0.31 / (np.log(E / E_CRIT))**0.5
        b = x
        c = 1.5 * x
        d = np.log((3 * x) / (x + 2 * np.log(E / E_CRIT)))
        nx = a * np.exp(b - c * d)

    else:  # hadronic shower profile
        # Gaisser-Hillas hadronic shower parameterization
        max_x = 200000.0 * units.g /units.cm**2 # maximum depth in g/cm^2
        dx = 1.0  * units.g /units.cm**2 # small enough bin in depth for our purposes.
        x_start = dx  # depth in g/cm^2
        S0 = 0.11842
        X0 = 39.562 * units.g /units.cm**2 # g/cm^2
        l = 113.03  * units.g /units.cm**2# g/cm^2
        Ec = 0.17006 * units.GeV  # GeV
        Xmax = X0 * np.log(E / Ec)
        x = np.arange(x_start, max_x, dx)
        a = S0 * E / Ec * (Xmax - l) / Xmax * np.exp(Xmax / l - 1)
        b = pow(x / (Xmax - l), Xmax / l)
        c = np.exp(-x / l)
        nx = a * b * c
    # find location of maximum, and charge excess from Fig. 5.9, compare in cm not m.
    n_max_position = np.argmax(nx)
    n_max = np.max(nx)
    if EM:
        excess = 0.09 + dx * n_max_position * ICE_RAD_LENGTH / ICE_DENSITY / 100.
    else:
        excess = 0.09 + dx * n_max_position / ICE_DENSITY * 1.0e-2
    Nmax = excess * n_max / 1000.0
    logger.debug("Nmax {}, excess {}, n_max {}".format(Nmax, excess, n_max))
    
    # We want to perform a fit for the regions with an excess charge 10% close to the maximum
    fit_region_cut = 0.95 
    cut_left = np.argwhere((nx[:n_max_position] / nx[n_max_position]) > fit_region_cut)[0][0]
    cut_right = np.argwhere((nx[n_max_position:] / nx[n_max_position]) < fit_region_cut)[0][0]+n_max_position
    fit_width = cut_right-cut_left
    max_vicinity = nx[n_max_position-fit_width:n_max_position+fit_width]/nx[n_max_position]
    x_fit = np.arange(0, len(max_vicinity), 1)
    sigma = curve_fit(gauss, x_fit, max_vicinity)[0]
    if EM:
        _askaryanDepthA = dx * sigma[2] / ICE_DENSITY * ICE_RAD_LENGTH 
    else:
        _askaryanDepthA = dx * sigma[2]/ ICE_DENSITY
    logger.debug("a (before LPM = {}".format(_askaryanDepthA))

    E_LPM = 3e14 * units.eV
    if(EM and LPM):
        if((E > E_LPM) or not fudge_LPM): # only apply LPM correction in regimes where it is relevant 
            p1 = -2.8564e2
            p2 = 7.8140e1
            p3 = -8.3893
            p4 = 4.4175e-1
            p5 = -1.1382e-2
            p6 = 1.1493e-4
            e = np.log10(E/units.eV)  # log_10 of Energy in eV
            log10_shower_depth = p1 + p2 * e + p3 * e**2 + p4 * e**3 + p5 * e**4 + p6 * e**5
            a = 10.0**log10_shower_depth * 0.5 # adjust shower wiedth to be just the sigma parameter of a Gaussian
            
            if(fudge_LPM):
                # normalize to Greisen parameterization at LPM energy
                a_Greisen = get_N_AskDepthA(E_LPM, EM=True, LPM=False)[1]
                a /= a_Greisen 
            
            # Right here, record the reduction in n_max_position that I don't believe in.
            if _strictLowFreqLimit:
                logger.debug("strict_lowfeq  Nmax = {:.2g}, a= {} priora = {}".format(Nmax, a, _askaryanDepthA/units.m, Nmax))
                Nmax = Nmax / (a / _askaryanDepthA)
            _askaryanDepthA = a
    logger.debug("a = {:.2f}m, Nmax = {}".format(_askaryanDepthA/units.m, Nmax))
    return Nmax, _askaryanDepthA






import numpy as np
import h5py

from NuRadioMC.EvtGen import generator
from NuRadioMC.utilities import units
import logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger("testtaueventgen")


# define simulation volume
fiducial_zmin = -2.7 * units.km
fiducial_zmax = 0 * units.km
generator.generate_eventlist_cylinder('tau.hdf5', 1e4, 1e18 * units.eV, 1e19 * units.eV,
                            0, 3*units.km, fiducial_zmin, fiducial_zmax, add_tau_second_bang=True)


print("writing many subfiles")
generator.generate_eventlist_cylinder('tau2.hdf5', 1e4, 1e16 * units.eV, 1e19 * units.eV,
                            0, 3*units.km, fiducial_zmin, fiducial_zmax, add_tau_second_bang=True, n_events_per_file=10)




from NuRadioMC.EvtGen import generator
from NuRadioMC.utilities import units
import numpy as np
import glob
import os
import sys


"""
This example creates job files for a pre simulation that saves all events that trigger a central station

"""

n_events_per_file = 300

try:
    software = sys.argv[0]
    # specify the path to the software directory (where NuRadioMC, NuRadioReco and radiotools are installed in)
    working_dir   = sys.argv[1]
    #where you want the simulation to go
    conf_dir   = sys.argv[2]  # where are the config files
    input_dir = sys.argv[3]
    # Directory with simulations ()

except:
    print("Usage python A02generate_job_files_presim.py software working_dir conf_dir input_dir")
    print("Using default values")
    conf_dir = "/pub/arianna/NuRadioMC/examples/03_station_coincidences/"
    working_dir = os.path.join(conf_dir, "presim")
    software = '/data/users/jcglaser/software'
    input_dir = '/pub/arianna/NuRadioMC/input_1e4/'

config_file = os.path.join(conf_dir, 'config.yaml')


if not os.path.exists(os.path.join(working_dir, "output")):
    os.makedirs(os.path.join(working_dir, "output"))
if not os.path.exists(os.path.join(working_dir, "run")):
    os.makedirs(os.path.join(working_dir, "run"))

for iI, filename in enumerate(sorted(glob.glob(os.path.join(input_dir, "*/*.hdf*")))):
    current_folder = os.path.split(os.path.dirname(filename))[-1]
    t1 = os.path.join(working_dir, "output", current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)
    t1 = os.path.join(working_dir, 'run', current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)

#     print('generating job submission files for {}'.format(filename))
    detector_file = os.path.join(conf_dir, 'single_position.json')
    output_filename = os.path.join(working_dir, "output", current_folder, os.path.basename(filename))
    cmd = "python {} {} {} {} {}\n".format(os.path.join(conf_dir, 'E06RunSimPreprocess.py'),
                                           filename, detector_file, config_file, output_filename)

    header = '#!/bin/bash\n'
    header += '#$ -N PreSim_{}\n'.format(iI)
    header += '#$ -j y\n'
    header += '#$ -V\n'
    header += '#$ -q grb,grb64\n'
#         header += '#$ -m \n'
    header += '#$ -o {}\n'.format(os.path.join(working_dir, 'run'))
    # add the software to the PYTHONPATH
    header += 'export PYTHONPATH={}/NuRadioMC:$PYTHONPATH\n'.format(software)
    header += 'export PYTHONPATH={}/NuRadioReco:$PYTHONPATH \n'.format(software)
    header += 'export PYTHONPATH={}/radiotools:$PYTHONPATH \n'.format(software)
    header += 'cd {} \n'.format(working_dir)

    with open(os.path.join(working_dir, 'run', current_folder, os.path.basename(filename) + ".sh"), 'w') as fout:
        fout.write(header)
        fout.write(cmd)



from __future__ import absolute_import, division, print_function
import logging
logging.basicConfig()

"""
Structure of a ray-tracing module. For documentation and development purposes.
"""

solution_types = {1: 'direct',
                  2: 'refracted',
                  3: 'reflected'}


class ray_tracing_base:
    """
    base class of ray tracer. All ray tracing modules need to prodide the following functions
    """

    def __init__(self, x1, x2, medium, attenuation_model="SP1", log_level=logging.WARNING,
                 n_frequencies_integration=6):
        """
        class initilization

        Parameters
        ----------
        x1: 3dim np.array
            start point of the ray
        x2: 3dim np.array
            stop point of the ray
        medium: medium class
            class describing the index-of-refraction profile
        attenuation_model: string
            signal attenuation model (so far only "SP1" is implemented)
        log_level: logging object
            specify the log level of the ray tracing class
            * logging.ERROR
            * logging.WARNING
            * logging.INFO
            * logging.DEBUG
            default is WARNING
        n_frequencies_integration: int
            the number of frequencies for which the frequency dependent attenuation
            length is being calculated. The attenuation length for all other frequencies
            is obtained via linear interpolation.

        """
        pass


    def find_solutions(self):
        """
        find all solutions between x1 and x2
        """
        pass

    def has_solution(self):
        """
        checks if ray tracing solution exists
        """
        return len(self.__results) > 0

    def get_number_of_solutions(self):
        """
        returns the number of solutions
        """
        return len(self.__results)

    def get_results(self):
        """

        """
        return self.__results

    def get_solution_type(self, iS):
        """ returns the type of the solution

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        Returns
        -------
        solution_type: int
            * 1: 'direct'
            * 2: 'refracted'
            * 3: 'reflected
        """
        pass

    def get_path(self, iS, n_points=1000):
        """
        helper function that returns the 3D ray tracing path of solution iS

        Parameters
        ----------
        iS: int
            ray tracing solution
        n_points: int
            number of points of path
        """
        pass


    def get_launch_vector(self, iS):
        """
        calculates the launch vector (in 3D) of solution iS

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        Returns
        -------
        launch_vector: 3dim np.array
            the launch vector
        """
        pass

    def get_receive_vector(self, iS):
        """
        calculates the receive vector (in 3D) of solution iS

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        Returns
        -------
        receive_vector: 3dim np.array
            the receive vector
        """
        pass

    def get_reflection_angle(self, iS):
        """
        calculates the angle of reflection at the surface (in case of a reflected ray)

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        Returns
        -------
        reflection_angle: float or None
            the reflection angle (for reflected rays) or None for direct and refracted rays
        """
        pass

    def get_path_length(self, iS, analytic=True):
        """
        calculates the path length of solution iS

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        analytic: bool
            If True the analytic solution is used. If False, a numerical integration is used. (default: True)

        Returns
        -------
        distance: float
            distance from x1 to x2 along the ray path
        """
        pass

    def get_travel_time(self, iS, analytic=True):
        """
        calculates the travel time of solution iS

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        analytic: bool
            If True the analytic solution is used. If False, a numerical integration is used. (default: True)

        Returns
        -------
        time: float
            travel time
        """
        pass

    def get_attenuation(self, iS, frequency, max_detector_freq=None):
        """
        calculates the signal attenuation due to attenuation in the medium (ice)

        Parameters
        ----------
        iS: int
            choose for which solution to compute the launch vector, counting
            starts at zero

        frequency: array of floats
            the frequencies for which the attenuation is calculated

        max_detector_freq: float or None
            the maximum frequency of the final detector sampling
            (the simulation is internally run with a higher sampling rate, but the relevant part of the attenuation length
            calculation is the frequency interval visible by the detector, hence a finer calculation is more important)

        Returns
        -------
        attenuation: array of floats
            the fraction of the signal that reaches the observer
            (only ice attenuation, the 1/R signal falloff not considered here)
        """
        pass




from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units


def get_ice_model(name):
    if(name == "ARAsim_southpole"):
        return ARAsim_southpole()
    elif(name == "southpole_simple"):
        return southpole_simple()
    elif(name == "southpole_2015"):
        return southpole_2015()
    elif(name == "mooresbay_simple"):
        return mooresbay_simple()
    elif(name == "greenland_simple"):
        return greenland_simple()

class medium():

    def get_index_of_refraction(self, x):
        """
        returns the index of refraction at position x

        Parameters
        ---------
        x: 3dim np.array
            point

        Returns:
        --------
        n: float
            index of refraction
        """
        return self.n_ice - self.delta_n * np.exp(x[2] / self.z_0)


class southpole_simple(medium):

    def __init__(self):
        # from https://doi.org/10.1088/1475-7516/2018/07/055 RICE2014/SP model
        # define model parameters (RICE 2014/southpole)
        self.n_ice = 1.78
        self.z_0 = 71. * units.m
        self.delta_n = 0.426

class southpole_2015(medium):

    def __init__(self):
        # from https://doi.org/10.1088/1475-7516/2018/07/055 SPICE2015/SP model
        self.n_ice = 1.78
        self.z_0 = 77. * units.m
        self.delta_n = 0.423

class ARAsim_southpole(medium):

    def __init__(self):
        # define model parameters (SPICE 2015/southpole)
        self.n_ice = 1.78
        self.z_0 = 75.75757575757576 * units.m
        self.delta_n = 0.43


class mooresbay_simple(medium):
    # from https://doi.org/10.1088/1475-7516/2018/07/055 MB1 model
    def __init__(self):
        self.n_ice = 1.78
        self.z_0 = 34.5 * units.m
        self.delta_n = 0.46
        self.reflection = -576 * units.m  # from https://doi.org/10.3189/2015JoG14J214
        self.reflection_coefficient = 0.82 # from https://doi.org/10.3189/2015JoG14J214
        self.reflection_phase_shift = 180 * units.deg
        
class mooresbay_simple_2(medium):
    # from https://doi.org/10.1088/1475-7516/2018/07/055 MB2 model
    def __init__(self):
        self.n_ice = 1.78
        self.z_0 = 37 * units.m
        self.delta_n = 0.481
        self.reflection = -576 * units.m  # from https://doi.org/10.3189/2015JoG14J214
        self.reflection_coefficient = 0.82 # from https://doi.org/10.3189/2015JoG14J214
        self.reflection_phase_shift = 180 * units.deg
        
        

class greenland_simple(medium):
    # from C. Deaconu, fit to data from Hawley '08, Alley '88
    # rho(z) = 917 - 602 * exp (-z/37.25), using n = 1 + 0.78 rho(z)/rho_0
    def __init__(self):
        self.n_ice = 1.78
        self.z_0 = 37.25 * units.m
        self.delta_n = 0.51


from __future__ import print_function, division
import time
import numpy as np
from NuRadioMC.utilities import units
import os
from scipy import interpolate as intp
import glob
import pickle
import sys
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.SignalGen.ARZ import ARZ
import logging
import matplotlib.gridspec as gridspec


logger = logging.getLogger("test")
logging.basicConfig(level=logging.WARNING)
rho = 0.924 * units.g / units.cm ** 3  # density g cm^-3

if __name__ == "__main__":
    T=100 * units.ns
    dt = 0.1*units.ns
    N = np.int(np.round(T/dt))
    tt = np.arange(0, dt*N, dt)
    n_index = 1.78
    theta = np.arccos(1./n_index)
    R = 1 * units.km
    a = ARZ.ARZ(library="/Users/cglaser/work/ARIANNA/data/ARZ/v1.1/library_v1.1.pkl", interp_factor=1, interp_factor2=1)
    
    # HAD 1e19
    shower_energy = 10*units.EeV
    shower_type = "HAD"
    n = 10
    thetas = np.linspace(theta - 2 * units.deg, theta + 10 *units.deg, n)
    
    a.set_interpolation_factor(0.1)
    a.set_interpolation_factor2(1000)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("0.1x interpolation, 1000x interpolation2, 10 showers = {:.2f}s".format(time.time() - t0))
    
    a.set_interpolation_factor(0.1)
    a.set_interpolation_factor2(100)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("0.1x interpolation, 100x interpolation2, 10 showers = {:.2f}s".format(time.time() - t0))
    
    a.set_interpolation_factor(1)
    a.set_interpolation_factor2(1)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("no interpolation, 10 showers = {:.2f}s".format(time.time() - t0))
  
    a.set_interpolation_factor(10)
    a.set_interpolation_factor2(1)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("10x interpolation, 10 showers = {:.2f}s".format(time.time() - t0))
      
    a.set_interpolation_factor(100)
    a.set_interpolation_factor2(1)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("100x interpolation, 10 showers = {:.2f}s".format(time.time() - t0))
    
    a.set_interpolation_factor(1)
    a.set_interpolation_factor2(10)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("10x interpolation2, 10 showers = {:.2f}s".format(time.time() - t0))
    
    a.set_interpolation_factor(1)
    a.set_interpolation_factor2(100)
    t0 = time.time()
    for i in range(n):
        trace = a.get_time_trace(shower_energy, thetas[i], N, dt, shower_type, n_index, R, iN=i)
    print("100x interpolation2, 10 showers = {:.2f}s".format(time.time() - t0))
    


#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
import os
from NuRadioMC.utilities import units
from NuRadioMC.EvtGen.generator import generate_eventlist_cylinder
import numpy as np
np.random.seed(0)

# define simulation volume
zmin = -2.7 * units.km  # the ice sheet at South Pole is 2.7km deep
zmax = 0 * units.km
rmin = 0 * units.km
rmax = 4 * units.km

path = os.path.dirname(os.path.abspath(__file__))

# generate one event list at 1e18 eV with 10000 neutrinos
generate_eventlist_cylinder(os.path.join(path,'1e18_full.hdf5'),
                            5e4, 1e18 * units.eV, 1e18 * units.eV,
                            rmin, rmax, zmin, zmax,
                            full_rmin=rmin, full_rmax=rmax,
                            full_zmin=zmin, full_zmax=zmax)


import numpy as np
from NuRadioMC.utilities import units
import os
from scipy import interpolate as intp
import glob
import pickle
import sys

rho = 0.924 * units.g / units.cm**3  # density g cm^-3

if __name__ == "__main__":
    print("usage: python A01preprocess_shower_library.py /path/to/library/ outputfilename")
    path = sys.argv[1]
    out = sys.argv[2]
    
    library = {}
    for subfolder in ["HAD", "EM"]:
        if(subfolder not in library):
            library[subfolder] = {}
        for file_e in sorted(glob.glob(os.path.join(path, subfolder, "*.t1005"))):
            filename = os.path.splitext(os.path.basename(file_e))[0]
            file_p = file_e[:-1] + "6"
            if(subfolder == "EM"):
                estr = filename.split("E")[1][1:]
                if(estr.startswith("0")):
                    E = float("0.{}".format(estr[1:])) * units.EeV
                else:
                    E = float("{}".format(estr)) * units.EeV
            else:
                estr = filename.split("Enu")[1].split("Esh")
                e1 = estr[0].split("EeV")[0]
                e2 = estr[1].split("EeV")[0]
                if(e2.startswith("0")):
                    E = float("0.{}".format(e2[1:])) * units.EeV
                else:
                    E = float("{}".format(e2)) * units.EeV
                    
                
            
            depth_e, N_e = np.loadtxt(file_e, unpack=True, usecols=(1, 2))
            depth_p, N_p = np.loadtxt(file_p, unpack=True, usecols=(1, 2))
            depth_e *= units.g / units.cm**2
            depth_p *= units.g / units.cm**2
            depth_e -= 1000 * units.g/units.cm**2  # all simulations have an artificial offset of 1000 g/cm^2
            depth_p -= 1000 * units.g/units.cm**2
            ce = N_e - N_p
            # sanity check if files electron and positron profiles are compatible
            if (not np.all(depth_e == depth_p)):
                raise ImportError("electron and positron profile have different depths")
            
            if(E not in library[subfolder]):
                library[subfolder][E] = {}
                library[subfolder][E]['depth'] = depth_e
                library[subfolder][E]['charge_excess'] = []

            library[subfolder][E]['charge_excess'].append(ce)
             
            
            if 0:
                from matplotlib import pyplot as plt
                fig, ax = plt.subplots(1, 1)
                ax.plot(depth_e / units.g * units.cm**2, ce)
                ax.set_title("{} E = {:.2g}eV".format(subfolder, E))
                ax.semilogy(True)
                ax.set_ylabel("charge excess")
                ax.set_ylim(10)
                ax.set_xlabel(r"shower depth [g/cm$^2$]")
                fig.tight_layout()
                fig.savefig(os.path.join(path, "plots", subfolder, "{}.png".format(filename)))
                plt.close(fig)
                continue
#                 plt.show()
                
#             length = depth_e / rho
#             zmax = length.max()
#             xnep = intp.interp1d(length, N_e - N_p, bounds_error=False, fill_value=0)
    with open(os.path.join(path, out), 'wb') as fout:
        pickle.dump(library, fout, protocol=2)


import numpy as np
import matplotlib
from matplotlib import pyplot as plt
import h5py
import glob
from scipy import interpolate
import json
import os
import sys

from NuRadioReco.utilities import units
from NuRadioReco.detector import detector
from NuRadioMC.utilities import fluxes
from NuRadioMC.utilities.Veff import get_Veff
from NuRadioMC.examples.Sensitivities import E2_fluxes3 as limits
plt.switch_backend('agg')




if __name__ == "__main__":

    path = 'output'
    if(len(sys.argv) == 1):
        print("no path specified, assuming that hdf5 files are in directory 'output'")
    else:
        path = sys.argv[1]

    energies, Veff, Veff_error, SNR, trigger_names, deposited  = get_Veff(path)
    sortmask = np.argsort(energies)


    # plot effective volume
    fig, ax = plt.subplots(1, 1, figsize=(6,6))
    ax.errorbar(energies[sortmask] / units.eV, Veff['all_triggers'][sortmask]/units.km**3 /units.sr,
                yerr=Veff_error['all_triggers'][sortmask]/units.km**3/units.sr, fmt='d-')
    ax.semilogx(True)
    ax.semilogy(True)
    ax.set_xlabel("neutrino energy [eV]")
    ax.set_ylabel("effective volume [km$^3$ sr]")
    fig.tight_layout()
    fig.savefig("Veff.pdf")



    # plot expected limit
    fig, ax = limits.get_E2_limit_figure(diffuse=True,show_grand_10k=True, show_grand_200k=False)
    labels = []
    labels = limits.add_limit(ax, labels, energies[sortmask], Veff['all_triggers'].T[sortmask],
                              100, 'NuRadioMC example', livetime=3*units.year, linestyle='-',color='blue',linewidth=3)
    leg = plt.legend(handles=labels, loc=2)
    fig.savefig("limits.pdf")
    plt.show()


from __future__ import absolute_import, division, print_function
import numpy as np
from NuRadioMC.utilities import units
from six import iterkeys, iteritems
from scipy import constants
from scipy.integrate import quad
from scipy.interpolate import interp1d
from scipy.optimize import fsolve
import h5py
from NuRadioMC.EvtGen.generator import write_events_to_hdf5, split_hdf5_input_file
import logging
logger = logging.getLogger("EventGen")
logging.basicConfig()

VERSION_MAJOR = 1
VERSION_MINOR = 1

def generate_my_events(filename, n_events):
    """
    Event generator skeleton

    Parameters
    ----------
    filename: string
        the output filename of the hdf5 file
    n_events: int
        number of events to generate
    """
    
    # first set the meta attributes
    attributes = {}
    n_events = int(n_events)
    attributes['n_events'] = n_events
    attributes['start_event_id'] = 0
    attributes['fiducial_rmin'] = 0
    attributes['fiducial_rmax'] = 1 * units.km
    attributes['fiducial_zmin'] = 0 * units.m
    attributes['fiducial_zmax'] = -2 * units.km
    attributes['rmin'] = 0
    attributes['rmax'] = 1 * units.km
    attributes['zmin'] = 0 * units.m
    attributes['zmax'] = -2 * units.km
    attributes['Emin'] = 1 * units.EeV
    attributes['Emax'] = 1 * units.EeV

    # now generate the events and fill all required data sets
    # here we fill all data sets with dummy values
    data_sets = {}
    data_sets["azimuths"] = np.ones(n_events)
    data_sets["zeniths"] = np.ones(n_events)
    data_sets["xx"] = np.ones(n_events)
    data_sets["yy"] = np.ones(n_events)
    data_sets["zz"] = np.ones(n_events)
    data_sets["event_ids"] = np.arange(n_events)
    data_sets["n_interaction"] = np.ones(n_events, dtype=np.int)

    # generate neutrino flavors randomly
    data_sets["flavors"] = np.array([12 for i in range(n_events)])
    data_sets["energies"] = np.ones(n_events) * 1 * units.EeV

    data_sets["interaction_type"] = np.ones(n_events, dtype='S2')
    data_sets["inelasticity"] = np.ones(n_events) *  0.5
    
    
    # write events to file
    write_events_to_hdf5(filename, data_sets, attributes)



"""
standard system of units
===================================

You should use the units defined in this file whenever you
have a dimensional quantity in your code.  For example,
write:

    ``s = 1.5 * units.km``

instead of:

    ``s = 1.5   # don't forget this is in km!``

The conversion factors defined in this file
convert your data into Auger base units, so that
all dimensional quantities in the code are in a
single system of units!  You can also
use the conversions defined here to, for example,
display data with the unit of your choice.  For example:

    ``print "s = " , s / units.mm, " mm"``


The base units are:
-------------------
   * meter                   (meter)
   * nanosecond              (nanosecond)
   * electron Volt           (eV)
   * positron charge         (eplus)
   * degree Kelvin           (kelvin)
   * the amount of substance (mole)
   * luminous intensity      (candela)
   * radian                  (radian)
   * steradian               (steradian)

The SI numerical value of the positron charge is defined here,
as it is needed for conversion factor : positron charge = eSI (coulomb)

Adapted from Offline, the reconstruction Framework of the Pierre Auger Collaboration.
This is a slightly modified version of the units definitions written by the Geant4 collaboration

"""

# Prefixes
yocto = 1e-24
zepto = 1e-21
atto = 1e-18
femto = 1e-15
pico = 1e-12
nano = 1e-9
micro = 1e-6
milli = 1e-3
centi = 1e-2
deci = 1e-1
deka = 1e+1
hecto = 1e+2
kilo = 1e+3
mega = 1e+6
giga = 1e+9
tera = 1e+12
peta = 1e+15
exa = 1e+18
zetta = 1e+21
yotta = 1e+24

# Length [L]
meter = 1
meter2 = meter * meter
meter3 = meter * meter * meter

millimeter = milli * meter
millimeter2 = millimeter * millimeter
millimeter3 = millimeter * millimeter * millimeter

centimeter = centi * meter
centimeter2 = centimeter * centimeter
centimeter3 = centimeter * centimeter * centimeter

kilometer = kilo * meter
kilometer2 = kilometer * kilometer
kilometer3 = kilometer * kilometer * kilometer

parsec = 3.0856775807e+16 * meter
kiloParsec = kilo * parsec
megaParsec = mega * parsec

micrometer = micro * meter
nanometer = nano * meter
angstrom = 1e-10 * meter
fermi = femto * meter

feet = 0.3048 * meter

barn = 1e-28 * meter2
millibarn = milli * barn
microbarn = micro * barn
nanobarn = nano * barn
picobarn = pico * barn

# symbols
mm = millimeter
mm2 = millimeter2
mm3 = millimeter3

cm = centimeter
cm2 = centimeter2
cm3 = centimeter3

m = meter
m2 = meter2
m3 = meter3

km = kilometer
km2 = kilometer2
km3 = kilometer3

# Angle
radian = 1
milliradian = milli * radian
degree = (3.14159265358979323846 / 180) * radian

steradian = 1

# symbols
rad = radian
mrad = milliradian
sr = steradian
deg = degree

# Time [T]
nanosecond = 1
nanosecond2 = nanosecond * nanosecond
second = giga * nanosecond
millisecond = milli * second
microsecond = micro * second
picosecond = pico * second
femtosecond = femto * second
minute = 60 * second
hour = 60 * minute
day = 24 * hour
year = 356 * day

hertz = 1 / second
kilohertz = kilo * hertz
megahertz = mega * hertz
gigahertz = giga * hertz

Hz = hertz
kHz = kilohertz
MHz = megahertz
GHz = gigahertz

# symbols
ns = nanosecond
ps = picosecond
fs = femtosecond
s = second
ms = millisecond

# Electric charge [Q]
eplus = 1  # positron charge
eSI = 1.602176462e-19  # positron charge in coulomb
coulomb = eplus / eSI  # coulomb = 6.24150 e+18*eplus

# Energy [E]
electronvolt = 1
megaelectronvolt = mega * electronvolt
kiloelectronvolt = kilo * electronvolt
gigaelectronvolt = giga * electronvolt
teraelectronvolt = tera * electronvolt
petaelectronvolt = peta * electronvolt
exaelectronvolt = exa * electronvolt
zettaelectronvolt = zetta * electronvolt

joule = electronvolt / eSI  # joule = 6.24150 e+12 * MeV

# symbols
eV = electronvolt
keV = kiloelectronvolt
MeV = megaelectronvolt
GeV = gigaelectronvolt
TeV = teraelectronvolt
PeV = petaelectronvolt
EeV = exaelectronvolt
ZeV = zettaelectronvolt

# Mass [E][T^2][L^-2]
kilogram = joule * second * second / (meter * meter)
gram = milli * kilogram
milligram = milli * gram

# symbols
kg = kilogram
g = gram
mg = milligram

# Power [E][T^-1]
watt = joule / second  # watt = 6.24150 e+3 * MeV/ns

# Force [E][L^-1]
newton = joule / meter  # newton = 6.24150 e+9 * MeV/mm

# Pressure [E][L^-3]
pascal = newton / m2  # pascal = 6.24150 e+3 * MeV/mm3
bar = 100000 * pascal  # bar    = 6.24150 e+8 * MeV/mm3
atmosphere = 101325 * pascal  # atm    = 6.32420 e+8 * MeV/mm3

# symbols
hPa = hecto * pascal

# Electric current [Q][T^-1]
ampere = coulomb / second  # ampere = 6.24150 e+9 * eplus/ns
milliampere = milli * ampere
microampere = micro * ampere
nanoampere = nano * ampere

# Electric potential [E][Q^-1]
megavolt = megaelectronvolt / eplus
kilovolt = milli * megavolt
volt = micro * megavolt
millivolt = milli * volt
microvolt = micro * volt

V = volt
mV = millivolt

# Electric resistance [E][T][Q^-2]
ohm = volt / ampere  # ohm = 1.60217e-16*(MeV/eplus)/(eplus/ns)

# Electric capacitance [Q^2][E^-1]
farad = coulomb / volt  # farad = 6.24150e+24 * eplus/Megavolt
millifarad = milli * farad
microfarad = micro * farad
nanofarad = nano * farad
picofarad = pico * farad

# Magnetic Flux [T][E][Q^-1]
weber = volt * second  # weber = 1000*megavolt*ns

# Magnetic Field [T][E][Q^-1][L^-2]
tesla = volt * second / meter2  # tesla =0.001*megavolt*ns/mm2
microtesla = micro * tesla

gauss = 1e-4 * tesla
kilogauss = deci * tesla

# Inductance [T^2][E][Q^-2]
henry = weber / ampere  # henry = 1.60217e-7*MeV*(ns/eplus)**2

# Temperature
kelvin = 1

# Amount of substance
mole = 1

# Activity [T^-1]
becquerel = 1 / second
curie = 3.7e+10 * becquerel

# Absorbed dose [L^2][T^-2]
gray = joule / kilogram

# Luminous intensity [I]
candela = 1

# Luminous flux [I]
lumen = candela * steradian

# Illuminance [I][L^-2]
lux = lumen / meter2

# Miscellaneous
fraction = 1
perCent = 0.01
percent = perCent
perThousand = 0.001
permil = perThousand
perMillion = 0.000001



import numpy as np
from NuRadioMC.utilities import units

model_to_int = {"SP1" : 1, "GL1" : 2, "MB1" : 3}

def fit_GL1(z):
    """
    Returns the attenuation length at 75 MHz as a function of depth for Greenland
    # Model for Greenland. Taken from DOI: https://doi.org/10.3189/2015JoG15J057

    Parameters
    ----------
    z: float
        depth in default units
    """

    fit_values = [1.16052586e+03, 6.87257150e-02, -9.82378264e-05,
                    -3.50628312e-07, -2.21040482e-10, -3.63912864e-14]
    min_length = 100 * units.m
    if(not hasattr(z, '__len__')):
        att_length = 0
    else:
        att_length = np.zeros_like(z)
    for power, coeff in enumerate(fit_values):
        att_length += coeff * z**power

    if (not hasattr(att_length, '__len__')):
        if ( att_length < min_length ):
            att_length = min_length
    else:
        att_length[ att_length < min_length ] = min_length
    return att_length

def get_temperature(z):
    """
    returns the temperature in Celsius as a function of depth for South Pole

    Parameters
    ----------
    z: float
        depth in default units
    """
    # from https://icecube.wisc.edu/~araproject/radio/#icetabsorption

    z2 = np.abs(z / units.m)
    return 1.83415e-09 * z2**3 + (-1.59061e-08 * z2**2) + 0.00267687 * z2 + (-51.0696)

def get_attenuation_length(z, frequency, model):
    """
    Get attenuation length in ice for different ice models

    Parameters
    ----------
    z: float
        depth in default units
    frequency: float
        frequency of signal in default units
    model: string
        Ice model for attenuation length
        SP1: South Pole model, see various compilation
        GL1: Greenland model, see https://arxiv.org/abs/1409.5413
        MB1: Moore's Bay Model, from 10.3189/2015JoG14J214 and
            Phd Thesis C. Persichilli (depth dependence)
    """
    if(model == "SP1"):
        t = get_temperature(z)
        f0 = 0.0001
        f2 = 3.16
        w0 = np.log(f0)
        w1 = 0.0
        w2 = np.log(f2)
        w = np.log(frequency / units.GHz)
        b0 = -6.74890 + t * (0.026709 - t * 0.000884)
        b1 = -6.22121 - t * (0.070927 + t * 0.001773)
        b2 = -4.09468 - t * (0.002213 + t * 0.000332)
        if(not hasattr(frequency, '__len__')):
            if (frequency < 1. * units.GHz):
                a = (b1 * w0 - b0 * w1) / (w0 - w1)
                bb = (b1 - b0) / (w1 - w0)
            else:
                a = (b2 * w1 - b1 * w2) / (w1 - w2)
                bb = (b2 - b1) / (w2 - w1)
        else:
            a = np.ones_like(frequency) * (b2 * w1 - b1 * w2) / (w1 - w2)
            bb = np.ones_like(frequency) * (b2 - b1) / (w2 - w1)
            a[frequency < 1. * units.GHz] = (b1 * w0 - b0 * w1) / (w0 - w1)
            bb[frequency < 1. * units.GHz] = (b1 - b0) / (w1 - w0)

        return 1. / np.exp(a + bb * w)
    elif(model == "GL1"):

        att_length_75 = fit_GL1(z/units.m)
        att_length_f = att_length_75 - 0.55 * units.m * (frequency/units.MHz - 75)

        min_length = 100 * units.m
        if(not hasattr(frequency, '__len__') and not hasattr(z, '__len__')):
            if (att_length_f < min_length):
                att_length_f = min_length
        else:
            att_length_f[ att_length_f < min_length ] = min_length

        return att_length_f
    elif(model == "MB1"):
        # 10.3189/2015JoG14J214 measured the depth-averaged attenuation length as a function of frequency
        # the derived parameterization assumed a reflection coefficient of 1
        # however a reflection coefficient of 0.82 was measured which results in a slight increase of the derived 
        # attenution length (correction factor below)
        R = 0.82
        d_ice = 576 * units.m
        att_length = 460 * units.m - 180 * units.m /units.GHz * frequency
        att_length *= (1 + att_length / (2 * d_ice) * np.log(R)) ** -1  # additional correction for reflection coefficient being less than 1.

        # The temperature dependence of the attenuation length is independent of the frequency dependence
        # the relationship between temparature and L is from 10.1063/1.363582
        # the temparature profile of the Ross Ice shelf is from 10.1126/science.203.4379.433 and rescaled to the shelf 
        # thickness of the ARIANNA site (almost linear from -28 - -2deg C. 
        # this theoretical depth dependence is scaled to match the depth-averaged measurement of the attenuation length.
        d = -z * 420. / d_ice;
        L = (1250.*0.08886 * np.exp(-0.048827 * (225.6746 - 86.517596 * np.log10(848.870 - (d)))))
        att_length *= L/262.0

        return att_length
    else:
        raise NotImplementedError("attenuation model {} is not implemented.".format(model))



from NuRadioMC.EvtGen.generator import *

filename = 'decay_library.hdf5'
fout = h5py.File(filename, 'w')

#times = np.linspace(1e-3*tau_rest_lifetime, 10*tau_rest_lifetime, 200)
times = np.linspace(-3,10,100)
times = 10**times * tau_rest_lifetime
energies = np.linspace(15, 20, 100)
energies = 10**energies * units.eV

# "Clever" way of looping. However, we can't see the progress with this.
#tables = [ [ get_decay_time_losses(energy, 1000*units.km, average=True, compare=True, user_time=time)
#            for time in times ] for energy in energies ]

tables = []
for itime, time in enumerate(times):
    row = []
    for ienergy, energy in enumerate(energies):
        print(itime, ienergy)
        row.append( get_decay_time_losses(energy, 1000*units.km, average=True, compare=False, user_time=time) )
    tables.append(row)

tables = np.array(tables)

fout['decay_times'] = tables[:,:,0]
fout['decay_energies'] = tables[:,:,1]
fout['rest_times'] = times
fout['initial_energies'] = energies

fout.close()



import logging
import numpy as np
import argparse
import matplotlib.pyplot as plt

from NuRadioReco.utilities import units
import NuRadioReco.detector.detector as detector
import NuRadioReco.modules.io.eventReader

from NuRadioReco.framework.parameters import stationParameters as stnp

logging.basicConfig(level=logging.INFO)

# Parse eventfile as argument
parser = argparse.ArgumentParser(description='NuRadioSim file')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC simulation result')
parser.add_argument('detectordescription', type=str,
                    help='path to detectordescription')
args = parser.parse_args()


# read in detector positions (this is a dummy detector)
det = detector.Detector(json_filename=args.detectordescription)

# initialize modules
eventReader = NuRadioReco.modules.io.eventReader.eventReader()
eventReader.begin(args.inputfilename)

for event in eventReader.run():
    for station in event.get_stations():
        station_id = station.get_id()
        for channel in station.iter_channels():
            channel_id = channel.get_id()
            
            # get time trace and times of bins
            trace = channel.get_trace()
            times = channel.get_times()
            
            # or get the frequency spetrum instead
            spectrum = channel.get_frequency_spectrum()
            frequencies = channel.get_frequencies()
            
            # obtain the position of the channel/antenna from the detector description
            position = det.get_relative_position(station_id, channel_id)
            
            
            



from __future__ import absolute_import, division, print_function
import numpy as np
from radiotools import helper as hp
from radiotools import plthelpers as php
from matplotlib import pyplot as plt
from NuRadioMC.utilities import units
from NuRadioMC.utilities import medium
from NuRadioMC.SignalProp import analyticraytraycing as ray
import h5py
import argparse
import json
import time
import os

parser = argparse.ArgumentParser(description='Plot NuRadioMC event list output.')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC hdf5 simulation output')
# parser.add_argument('outputfilename', type=str,
#                     help='name of output file storing the electric field traces at detector positions')
args = parser.parse_args()

fin = h5py.File(args.inputfilename, 'r')

weights = np.array(fin['weights'])
triggered = np.array(fin['triggered'])
n_events = fin.attrs['n_events']

# plot vertex distribution
fig, ax = plt.subplots(1, 1)
xx = np.array(fin['xx'])
yy = np.array(fin['yy'])
rr = (xx ** 2 + yy ** 2) ** 0.5
zz = np.array(fin['zz'])
h = ax.hist2d(rr / units.m, zz / units.m, bins=[np.arange(0, 4000, 100), np.arange(-3000, 0, 100)],
              cmap=plt.get_cmap('Blues'), weights=weights)
cb = plt.colorbar(h[3], ax=ax)
cb.set_label("weighted number of events")
ax.set_aspect('equal')
ax.set_xlabel("r [m]")
ax.set_ylabel("z [m]")
fig.tight_layout()

mask = (zz < -2000 * units.m) & (rr < 5000 * units.m)
for i in np.array(range(len(xx)))[mask]:
#     C0 = fin['ray_tracing_C0'][i][0][0]
#     C1 = fin['ray_tracing_C1'][i][0][0]
    print('weight = {:.2f}'.format(weights[i]))
    x1 = np.array([xx[i], yy[i], zz[i]])
    x2 = np.array([0, 0, -5])
    r = ray.ray_tracing(x1, x2, medium.southpole_simple())
    r.find_solutions()
    C0 = r.get_results()[0]['C0']
    x1 = np.array([-rr[i], zz[i]])
    x2 = np.array([0, -5])
    r2 = ray.ray_tracing_2D(medium.southpole_simple())
    yyy, zzz = r2.get_path(x1, x2, C0)

    launch_vector = fin['launch_vectors'][i][0][0]
    print(launch_vector)
    zenith_nu = fin['zeniths'][i]
    print(zenith_nu / units.deg)

    fig, ax = plt.subplots(1, 1)
    ax.plot(-rr[i], zz[i], 'o')
    ax.plot([-rr[i], -rr[i] + 100 * np.cos(zenith_nu)], [zz[i], zz[i] + 100 * np.sin(zenith_nu)], '-C0')
    ax.plot(0, -5, 'd')
    ax.plot(yyy, zzz)
    ax.set_aspect('equal')
    plt.show()




import numpy as np
import pickle
import time
from NuRadioMC.SignalProp import analyticraytracing as ray
from NuRadioMC.utilities import units, medium
import logging
from numpy import testing
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('test_raytracing')

ice = medium.mooresbay_simple()

np.random.seed(10)  # set seed to have reproducible results
n_events = int(1e3)
rmin = 50. * units.m
rmax = 3. * units.km
zmin = 0. * units.m
zmax = -0.5 * units.km
rr = np.random.triangular(rmin, rmax, rmax, n_events)
phiphi = np.random.uniform(0, 2 * np.pi, n_events)
xx = rr * np.cos(phiphi)
yy = rr * np.sin(phiphi)
zz = np.random.uniform(zmin, zmax, n_events)

points = np.array([xx, yy, zz]).T
x_receiver = np.array([0., 0., -5.])

results_C0s_cpp = np.zeros((n_events, 10))
n_freqs = 256/2 + 1
# n_freqs = 5
results_A_cpp = np.zeros((n_events, 2, n_freqs))
t_start = time.time()
ff = np.linspace(0, 500*units.MHz, n_freqs)
# tt = 0
for iX, x in enumerate(points):
#     t_start2 = time.time()
    r = ray.ray_tracing(x, x_receiver, ice, n_reflections=2)
#     tt += (time.time() - t_start2)
    r.find_solutions()
    if(r.has_solution()):
        for iS in range(r.get_number_of_solutions()):
            results_C0s_cpp[iX, iS] = r.get_results()[iS]['C0']
with open("reference_C0_MooresBay.pkl", "w") as fout:
    pickle.dump(results_C0s_cpp, fout, protocol=2)




from NuRadioMC.EvtGen import generator
from NuRadioMC.utilities import units
import numpy as np
import glob
import os
import sys


"""
This example creates job files for simulations to check for coincidences.

"""

n_events_per_file = 300

try:
    software = sys.argv[0]
    # specify the path to the software directory (where NuRadioMC, NuRadioReco and radiotools are installed in)
    working_dir   = sys.argv[1]
    #where you want the simulation to go
    conf_dir   = sys.argv[2]  # where are the config files
    presim_dir = sys.argv[3]
    # Directory with simulations ()

except:
    print("Usage python A02generate_job_files.py software working_dir presim_dir")
    print("Using default values")
    conf_dir = "/pub/arianna/NuRadioMC/examples/03_station_coincidences/"
    working_dir = conf_dir
    software = '/data/users/jcglaser/software'
    presim_dir = os.path.join(working_dir, 'presim/output')

config_file = os.path.join(working_dir, 'config.yaml')


if not os.path.exists(os.path.join(working_dir, "output")):
    os.makedirs(os.path.join(working_dir, "output"))
if not os.path.exists(os.path.join(working_dir, "run")):
    os.makedirs(os.path.join(working_dir, "run"))
if not os.path.exists(os.path.join(working_dir, "input")):
    os.makedirs(os.path.join(working_dir, "input"))

for iI, filename in enumerate(sorted(glob.glob(os.path.join(presim_dir,"*.hdf5")))):
    current_folder = os.path.splitext(os.path.basename(filename))[0]
    t1 = os.path.join(working_dir, "output", current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)
    t1 = os.path.join(working_dir, 'run', current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)
    t1 = os.path.join(working_dir, 'input', current_folder)
    if(not os.path.exists(t1)):
        os.makedirs(t1)

    output_filename = os.path.join(working_dir, 'input', current_folder, os.path.basename(filename))
    
    print('saving files to {}'.format(output_filename))

    generator.split_hdf5_input_file(filename, output_filename, n_events_per_file)

    for iF, filename2 in enumerate(sorted(glob.glob(os.path.join(working_dir, 'input', current_folder, "{}*".format(os.path.basename(output_filename[:-1])))))):
        print('generating job submission files for {}'.format(filename2))
        detector_file = os.path.join(working_dir, 'horizontal_spacing_detector.json')
        output_filename = os.path.join("output", current_folder, os.path.basename(filename2))
        cmd = "python {} {} {} {} {}\n".format(os.path.join(working_dir, 'E06RunSimPreprocess.py'), filename2,
                                               detector_file, config_file, output_filename)

        header = '#!/bin/bash\n'
        header += '#$ -N HS_{}_{}\n'.format(iI, iF)
        header += '#$ -j y\n'
        header += '#$ -V\n'
        header += '#$ -q grb,grb64\n'
#         header += '#$ -m \n'
        header += '#$ -o {}\n'.format(os.path.join(working_dir, 'run'))
        # add the software to the PYTHONPATH
        header += 'export PYTHONPATH={}/NuRadioMC:$PYTHONPATH\n'.format(software)
        header += 'export PYTHONPATH={}/NuRadioReco:$PYTHONPATH \n'.format(software)
        header += 'export PYTHONPATH={}/radiotools:$PYTHONPATH \n'.format(software)
        header += 'cd {} \n'.format(working_dir)

        with open(os.path.join('run', current_folder, os.path.basename(filename2) + ".sh"), 'w') as fout:
            fout.write(header)
            fout.write(cmd)



from __future__ import absolute_import, division, print_function
import argparse
from six import iteritems
# import detector simulation modules
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelGenericNoiseAdder
import NuRadioReco.modules.custom.deltaT.calculateAmplitudePerRaySolution
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation
from NuRadioReco.framework.parameters import electricFieldParameters as efp
import logging
import numpy as np
logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger("runDeltaTStudy")

# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin(pre_pulse_time=0 * units.ns, post_pulse_time=0 * units.ns)
calculateAmplitudePerRaySolution = NuRadioReco.modules.custom.deltaT.calculateAmplitudePerRaySolution.calculateAmplitudePerRaySolution()
triggerSimulator = NuRadioReco.modules.trigger.simpleThreshold.triggerSimulator()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()


class mySimulation(simulation.simulation):


    def _detector_simulation(self):
        if(bool(self._cfg['signal']['zerosignal'])):
            self._increase_signal(None, 0)

        calculateAmplitudePerRaySolution.run(self._evt, self._station, self._det)
        # save the amplitudes to output hdf5 file
        # save amplitudes per ray tracing solution to hdf5 data output
        sg = self._mout_groups[self._station_id]
        n_antennas = self._det.get_number_of_channels(self._station_id)
        if('max_amp_ray_solution' not in sg):
            sg['max_amp_ray_solution'] = np.zeros((self._n_events, n_antennas, 2)) * np.nan
        ch_counter = np.zeros(n_antennas, dtype=np.int)
        for efield in self._station.get_sim_station().get_electric_fields():
            for channel_id, maximum in iteritems(efield[efp.max_amp_antenna]):
                sg['max_amp_ray_solution'][self._iE, channel_id, ch_counter[channel_id]] = maximum
                ch_counter[channel_id] += 1 
        
        # start detector simulation
        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        
        # downsample trace to internal simulation sampling rate (the efieldToVoltageConverter upsamples the trace to
        # 20 GHz by default to achive a good time resolution when the two signals from the two signal paths are added) 
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=1. / self._dt)
        
        if bool(self._cfg['noise']):
            Vrms = self._Vrms / (self._bandwidth /( 2.5 * units.GHz))** 0.5  # normalize noise level to the bandwidth its generated for
            channelGenericNoiseAdder.run(self._evt, self._station, self._det, amplitude=Vrms, min_freq=0 * units.MHz,
                                         max_freq=2.5 * units.GHz, type='rayleigh')

        # bandpass filter trace, the upper bound is higher then the sampling rate which makes it just a highpass filter
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[80 * units.MHz, 1000 * units.GHz],
                                  filter_type='butter', order=2)
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[0, 500 * units.MHz],
                                  filter_type='butter', order=10)
        triggerSimulator.run(self._evt, self._station, self._det,
                             threshold=2 * self._Vrms,
                             triggered_channels=None,
                             number_concidences=1,
                             trigger_name='pre_trigger_2sigma')
        
        # downsample trace back to detector sampling rate
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=self._sampling_rate_detector)


parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC input event list')
parser.add_argument('detectordescription', type=str,
                    help='path to file containing the detector description')
parser.add_argument('config', type=str,
                    help='NuRadioMC yaml config file')
parser.add_argument('outputfilename', type=str,
                    help='hdf5 output filename')
parser.add_argument('outputfilenameNuRadioReco', type=str, nargs='?', default=None,
                    help='outputfilename of NuRadioReco detector sim file')
args = parser.parse_args()

sim = mySimulation(eventlist=args.inputfilename,
                            outputfilename=args.outputfilename,
                            detectorfile=args.detectordescription,
                            outputfilenameNuRadioReco=args.outputfilenameNuRadioReco,
                            config_file=args.config)
sim.run()



import numpy as np
from NuRadioMC.utilities import units
from NuRadioReco.utilities import io_utilities
import os
from scipy import interpolate as intp
import glob
import sys
from matplotlib import pyplot as plt

if __name__ == "__main__":
    library_path = sys.argv[1]
    lib = io_utilities.read_pickle(library_path)
    
    f, ax = plt.subplots(1, len(lib), sharey=True)
    
    for iS, shower_type in enumerate(lib):
        b = []
        nb = []
        for E in lib[shower_type]:
            nE = len(lib[shower_type][E]['charge_excess'])
            b.append(E)
            nb.append(nE)

        ax[iS].bar(np.log10(b), nE, linewidth=1, edgecolor='k', width=0.05)
        ax[iS].set_xlabel("log10(shower energy [eV])")
        ax[iS].set_title(shower_type)
    ax[0].set_ylabel('number of showers')
    f.tight_layout()
    plt.show()
            
        


import numpy as np
from matplotlib import pyplot as plt
from NuRadioMC.utilities import fluxes
from NuRadioMC.examples.Sensitivities import E2_fluxes2 as limit
from NuRadioMC.utilities import units
import json


# import latest Veff from NuRadioMC
with open("NuRadioMC_20181113_Veff.json", 'r') as fin:
    data = json.load(fin)
    
    # producen nice outreach plot
    fig, ax = limit.get_E2_limit_figure()
    labels = []
    labels = limit.add_limit(ax, labels,
                             data['shallow + PA@15m@2s']['energies'],
                             data['shallow + PA@15m@2s']['Veff'], 200, 'GIANT', fmt='-C1')
    labels = limit.add_limit(ax, labels,
                             data['shallow + PA@15m@2s']['energies'],
                             data['shallow + PA@15m@2s']['Veff'], 2000, 'GIANT', fmt='-C3')
    plt.legend(handles=labels, loc=2)
    ax.set_title("raw triggered volumes")
    fig.tight_layout()
    fig.savefig("limits_outreach.pdf")
    
    
    fig, ax = limit.get_E2_limit_figure()
    labels = []
    labels = limit.add_limit(ax, labels,
                             data['shallow + PA@15m@2s']['energies'],
                             data['shallow + PA@15m@2s']['Veff'], 270, 'shallow + PA@15m', fmt='-C0')
    labels = limit.add_limit(ax, labels, data['shallow + PA@50m@2s']['energies'],
                             data['shallow + PA@50m@2s']['Veff'], 130, 'shallow + deep 60m', fmt='--C0')
    labels = limit.add_limit(ax, labels, data['shallow + PA@15m@2s']['energies'],
                             data['shallow + PA@15m@2s']['Veff'], 110, 'shallow + PA@15m', fmt='-C1')
    labels = limit.add_limit(ax, labels, data['shallow + PA@50m@2s']['energies'],
                             data['shallow + PA@50m@2s']['Veff'], 55, 'shallow + deep 60m', fmt='--C1')
    plt.legend(handles=labels, loc=2)
    ax.set_title("raw triggered volumes")
    fig.tight_layout()
    fig.savefig("limits_shallow_deep60m_raw.pdf")

    fig, ax = limit.get_E2_limit_figure()
    labels = []
    labels = limit.add_limit(ax, labels, data['shallow + PA@15m@2s + 3x3s']['energies'],
                             data['shallow + PA@15m@2s + 3x3s']['Veff'], 270,
                             'shallow + PA@15m', fmt='-C0')
    labels = limit.add_limit(ax, labels, data['shallow + PA@50m@2s + 3x3s']['energies'],
                             data['shallow + PA@50m@2s + 3x3s']['Veff'], 130, 'shallow + deep 60m', fmt='--C0')
    labels = limit.add_limit(ax, labels, data['shallow + PA@15m@2s + 3x3s']['energies'],
                             data['shallow + PA@15m@2s + 3x3s']['Veff'], 110, 'shallow + PA@15m', fmt='-C1')
    labels = limit.add_limit(ax, labels, data['shallow + PA@50m@2s + 3x3s']['energies'],
                             data['shallow + PA@50m@2s + 3x3s']['Veff'], 55, 'shallow + deep 60m', fmt='--C1')
    plt.legend(handles=labels, loc=2)
    ax.set_title("with 3x 3sigma cut")
    fig.tight_layout()
    fig.savefig("limits_shallow_deep60m_reco.pdf")
    
    ## calculate number of neutrinos from icecube (nu mu) flux
    def print_N_neutrinos(trigger_name, n_stations):
        print("{} for 3 years and {} stations with 100% uptime".format(trigger_name, n_stations))
        E = np.array(data[trigger_name]['energies'])
        Nnu = fluxes.get_number_of_events_for_flux(E, limit.ice_cube_nu_fit(E),
                                                   data[trigger_name]['Veff'], 3 * units.year * n_stations)
        print("{:>10}: {}".format("energy [eV]", "N_nu"))
        for iE in range(len(E)):
            print("{:>10.1g}: {:.1f}".format(E[iE], Nnu[iE]))
        print("--------------------")
        print("{:>10}: {:.1f}".format("total", np.sum(Nnu)))
    
    print("number of neutrinos for icecube (numu) flux")
    print_N_neutrinos('shallow + PA@15m@2s', 270)
    print_N_neutrinos('shallow + PA@15m@2s', 110)
    print_N_neutrinos('shallow + PA@50m@2s', 130)
    print_N_neutrinos('shallow + PA@50m@2s', 55)
    
    plt.show()



from __future__ import absolute_import, division, print_function
import numpy as np
import argparse
from radiotools import helper as hp
from NuRadioMC.utilities import units
from io import BytesIO
import logging
logger = logging.getLogger("readEventList")

VERSION = 0.2


def read_eventlist(filename):
    version = 0
    with open(filename, 'r') as fin:
        lines = fin.readlines()
        for i, line in enumerate(lines):
            if line.startswith("VERSION"):
                version = float(line.split("=")[1])
                break
        if(version != VERSION):
            print("file version is {}. version != {} not supported".format(version, VERSION))
            raise NotImplementedError

        fin.seek(0)  # reset file to beginning
        data = np.genfromtxt(fin, comments='#', skip_header=1,
                             dtype=[('eventId', int), ('nuflavor', int),
                                    ('pnu', float),
                                    ('currentint', '|S2'),
                                    ('x', float), ('y', float),
                                    ('z', float), ('theta', float),
                                    ('phi', float), ('inelasticity', float)])
        return data


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Parse NuRadioMC event list.')
    parser.add_argument('filename', type=str,
                        help='path to NuRadioMC event list')
    args = parser.parse_args()
    events = read_eventlist(args.filename)



import numpy as np
import h5py
from NuRadioReco.detector import detector
from NuRadioMC.utilities import units
from scipy import constants
from matplotlib import pyplot as plt
import glob
from radiotools import plthelpers as php
import sys
from radiotools import helper as hp
import os
from NuRadioMC.SignalProp import analyticraytracing as ray
from NuRadioMC.utilities import medium
from mpl_toolkits import mplot3d
import matplotlib
import logging

Tnoise = 300
Vrms = (Tnoise * 50 * constants.k * 500 * units.MHz / units.Hz) ** 0.5

plot=False
counter = -1
filename = sys.argv[1]
x = float(sys.argv[2]) * units.m
print("using file {} and distance {:.0f}m".format(filename, x/units.m))

fin = h5py.File(filename)
print(np.log10(fin.attrs['Emin']))

with open('det.json', 'w') as fout:
    fout.write(fin.attrs['detector'])
    fout.close()
det = detector.Detector(json_filename="det.json")
max_amps_env = np.array(fin['station_101']['maximum_amplitudes_envelope'])
weights = fin['weights']

n_channels = det.get_number_of_channels(101)
n_stations = det.get_number_of_channels(101) / 4  # (we had 4 antennas per station)
xs = np.zeros(n_stations)
ys = np.zeros(n_stations)

triggered_near_surface = np.any(max_amps_env[:, 0:3] > (3 * Vrms), axis=1)  # triggered any LPDA or dipole of the center station
triggered_near_deep = max_amps_env[:, 3] > (3 * Vrms) # triggered deep dipole
triggered_surface = np.zeros((max_amps_env.shape[0], n_stations)) # create empy array of shape (n events, n stations)
triggered_deep = np.zeros((max_amps_env.shape[0], n_stations)) # create empy array of shape (n events, n stations)
# loop through all stations with different distances 
for i in range(n_stations):
    # select the 2 LPDA + 1 dipole channel and check if they fulfill the trigger condition
    triggered_surface[:, i] = np.any(max_amps_env[:, i * 4:(i * 4 + 3)] > (3 * Vrms), axis=1)
    triggered_deep[:, i] = max_amps_env[:, i * 4 + 3] > (3 * Vrms)
    # get their position
    xs[i] = np.abs(det.get_relative_position(101, i * 4)[0])
    ys[i] = np.abs(det.get_relative_position(101, i * 4)[1])

# loop through all simulated distances
coincidence_fractions_surface = np.zeros(len(np.unique(xs)))
coincidence_fractions_deep = np.zeros(len(np.unique(xs)))

med = medium.ARAsim_southpole()


mask = ((np.abs(xs) == x) & (np.abs(ys) == x))  \
    | ((np.abs(xs) == x) & (ys == 0)) \
    | ((np.abs(ys) == x) & (xs == 0))
    
theta  = np.arccos(1. / 1.78)
    
n_events = len(triggered_near_surface)
dTs = []
for iE in np.arange(n_events, dtype=np.int)[np.any(triggered_deep[:, mask], axis=1) & triggered_near_deep]:
    if(iE % 100 == 0):
        print(iE)
    if(plot):
        ax = plt.axes(projection='3d')
#     x1 = det.get_relative_position(101, 3)
#     ax.plot([x1[0]], [x1[1]], [x1[2]], 'ko')
    for j in np.append([0], np.arange(n_stations)[mask]):
        iC = j * 4 + 3
        x2 = det.get_relative_position(101, iC)
        if(plot):
            ax.plot([x2[0]], [x2[1]], [x2[2]], 'ko')
        if(j != 0 and (~(np.array(triggered_deep[iE], dtype=np.bool) & mask))[j]):
            continue
        vertex = np.array([fin['xx'][iE], fin['yy'][iE], fin['zz'][iE]])
#         print(fin.keys())
#         l1 = fin['launch_vectors'][iE][3] / units.deg
        l2 = fin['station_101']['launch_vectors'][iE][j*4 + 3]
        
#         r1 = ray.ray_tracing(vertex, x1, med, log_level=logging.DEBUG)
# #         r1.find_solutions()
#         r1.set_solution(fin['ray_tracing_C0'][iE][3], fin['ray_tracing_C1'][iE][3], fin['ray_tracing_solution_type'][iE][3])
#         path1 = r1.get_path(0)
        
        zen, az = fin['zeniths'][iE], fin['azimuths'][iE]
        v = hp.spherical_to_cartesian(zen, az)
        
        if(plot):
            r2 = ray.ray_tracing(vertex, x2, med, log_level=logging.INFO)
            r2.set_solution(fin['station_101']['ray_tracing_C0'][iE][iC], fin['station_101']['ray_tracing_C1'][iE][iC], fin['station_101']['ray_tracing_solution_type'][iE][iC])
            path2 = r2.get_path(0)
    #         ax.plot3D(path1.T[0], path1.T[1], path1.T[2], label='path 1')
            ax.plot3D(path2.T[0], path2.T[1], path2.T[2], label='path {}'.format(j))
            ax.plot3D([vertex[0], vertex[0] + 500*v[0]], [vertex[1], vertex[1] + 500*v[1]], [vertex[2], vertex[2] + 500*v[2]],
                      '--', label='shower direction')
        
        dT = []
        for l in l2:
#             print("{:.1f}".format((theta - hp.get_angle(-v, l))/units.deg))
            dT.append((theta - hp.get_angle(-v, l))/units.deg)
        dTs.append(np.min(np.abs(np.array(dT))))
        if(plot):
            R3 = hp.get_rotation(np.array([0, 0, 1]), -v)
            
            for phi in np.linspace(0, 2 * np.pi, 200):
                
                l = hp.spherical_to_cartesian(theta, phi)
                
    #             zen, az = hp.cartesian_to_spherical(v[0], v[1], v[2])
                R1 = np.array([[np.cos(az), -np.sin(az), 0], [np.sin(az), np.cos(az), 0], [0, 0, 1]])
                R2 = np.array([[np.cos(zen), 0, -np.sin(zen)], [0, 1, 0], [np.sin(zen), 0, np.cos(zen)]])
    #             l2 = np.matmul(R1, np.matmul(R2,l))
                l2 = np.matmul(R3, l)
    #             l2 = np.matmul(R2, np.matmul(R1, l))
                l2 *= 400
#                 print(hp.get_angle(v, l2)/units.deg)
                l2 += vertex
                ax.plot3D([vertex[0], l2[0]], [vertex[1], l2[1]], [vertex[2], l2[2]], 'C3-', alpha=0.2)
    if(plot):
        ax.legend()
    #     ax.set_aspect('equal')
        ax.set_xlabel("x [m]")
        ax.set_ylabel("y [m]")
        ax.set_zlabel("z [m]")
    #     plt.ion()
        plt.show()
#     a =     1/0
php.get_histogram(np.array(dTs))
plt.show()
        




from distutils.core import setup
from Cython.Build import cythonize
from distutils.extension import Extension
from Cython.Distutils import build_ext
import numpy
import os

try:
    print('Your $GSLDIR = ' + str(os.environ['GSLDIR']))
except:
    print('You have either not installed GSL or have not set the system variable $GSLDIR.\
           See NuRadioMC wiki for further GSL details. ')

extensions = [
    Extension('wrapper', ['wrapper.pyx'],
              #include_dirs=[numpy.get_include(), '../../utilities/'],
              include_dirs=[numpy.get_include(), '../../utilities/', str(os.environ['GSLDIR']) + '/include/'],
              extra_compile_args=['-O3'],
              libraries=['gsl', 'gslcblas'],
              language='c++'
              ),
]

setup(
    ext_modules=cythonize(extensions),
    cmdclass = {'build_ext': build_ext}
#     extra_compile_args=["-w", '-g']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division, print_function
import numpy as np
from numpy import testing
import time
from NuRadioMC.SignalProp import analyticraytracing as ray
from NuRadioMC.utilities import units, medium
import logging
logging.basicConfig(level=logging.INFO)

ice = medium.mooresbay_simple()

"""
this unit test compares the numerical and analytic calculation of path length and travel time for the Moore's Bay site with additional reflections off the bottom.
the numerical integration should be better than the analytic formula. For both calculations, the python version is used.
"""


np.random.seed(0)  # set seed to have reproducible results
n_events = int(1e3)
rmin = -10. * units.m
rmax = 1. * units.km
zmin = 0. * units.m
zmax = -500. * units.m
rr = np.random.triangular(rmin, rmax, rmax, n_events)
phiphi = np.random.uniform(0, 2 * np.pi, n_events)
xx = rr * np.cos(phiphi)
yy = rr * np.sin(phiphi)
zz = np.random.uniform(zmin, zmax, n_events)

points = np.array([xx, yy, zz]).T
x_receiver = np.array([0., 0., -5.])

nsol = 6
results_D = np.zeros((n_events, nsol))
results_D_analytic = np.zeros((n_events, nsol))
results_T = np.zeros((n_events, nsol))
results_T_analytic = np.zeros((n_events, nsol))

d_numeric = 0
d_analytic = 0
t_numeric = 0
t_analytic = 0
for iX, x in enumerate(points):
    r = ray.ray_tracing(x, x_receiver, ice, log_level=logging.WARNING, n_reflections=1)
    r.find_solutions()
    if(r.has_solution()):
        for iS in range(r.get_number_of_solutions()):
            t_start = time.time()
            results_D[iX, iS] = r.get_path_length(iS, analytic=False)
            d_numeric += time.time() - t_start
            t_start = time.time()
            results_D_analytic[iX, iS] = r.get_path_length(iS, analytic=True)
            d_analytic += time.time() - t_start
            
            
            t_start = time.time()
            results_T[iX, iS] = r.get_travel_time(iS, analytic=False)
            t_numeric += time.time() - t_start
            t_start = time.time()
            results_T_analytic[iX, iS] = r.get_travel_time(iS, analytic=True)
            t_analytic += time.time() - t_start
            
print("asserting travel times")
mask = results_T != 0
print("average deviation = {:.4f}ns".format(np.mean(results_T[mask] - results_T_analytic[mask])/units.ns))
testing.assert_allclose(results_T, results_T_analytic, atol=1e-2 * units.ns, rtol=1e-3)
print("asserting distances")
testing.assert_allclose(results_D, results_D_analytic, atol=4e-2 * units.m, rtol=1e-3)
print('T04MooresBay passed without issues')


import matplotlib.pyplot as plt
import numpy as np
import time
from numpy import testing
from NuRadioMC.SignalProp import analyticraytracing as ray
from NuRadioMC.utilities import units, medium
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('test_raytracing')

"""
this unit test compares the numerical and analytic calculation of path length and travel time,
the numerical integration should be better than the analytic formula. For both calculations, the python version is used.
"""


ice = medium.southpole_simple()

np.random.seed(0)  # set seed to have reproducible results
n_events = int(1e3)
rmin = 50. * units.m
rmax = 3. * units.km
zmin = 0. * units.m
zmax = -3. * units.km
rr = np.random.triangular(rmin, rmax, rmax, n_events)
phiphi = np.random.uniform(0, 2 * np.pi, n_events)
xx = rr * np.cos(phiphi)
yy = rr * np.sin(phiphi)
zz = np.random.uniform(zmin, zmax, n_events)

points = np.array([xx, yy, zz]).T
x_receiver = np.array([0., 0., -5.])

results_D = np.zeros((n_events, 2))
results_D_analytic = np.zeros((n_events, 2))
results_T = np.zeros((n_events, 2))
results_T_analytic = np.zeros((n_events, 2))


d_numeric = 0
d_analytic = 0
t_numeric = 0
t_analytic = 0
for iX, x in enumerate(points):
    r = ray.ray_tracing(x, x_receiver, ice)
    r.find_solutions()
    if(r.has_solution()):
        for iS in range(r.get_number_of_solutions()):
            t_start = time.time()
            results_D[iX, iS] = r.get_path_length(iS, analytic=False)
            d_numeric += time.time() - t_start
            t_start = time.time()
            results_D_analytic[iX, iS] = r.get_path_length(iS, analytic=True)
            d_analytic += time.time() - t_start
            
            
            t_start = time.time()
            results_T[iX, iS] = r.get_travel_time(iS, analytic=False)
            t_numeric += time.time() - t_start
            t_start = time.time()
            results_T_analytic[iX, iS] = r.get_travel_time(iS, analytic=True)
            t_analytic += time.time() - t_start
#             print("analytic {:.2f}".format(results_D_analytic[iX, iS]))
#             if(np.abs(results_D[iX, iS] - results_D_analytic[iX, iS]) > 0.1):
#                 print(np.abs(results_D[iX, iS] - results_D_analytic[iX, iS]))
#                 a = 1/0
t_cpp = time.time() - t_start
print("analytic {:.1f} seconds = {:.2f}ms/event".format(d_analytic, 1000. * d_analytic / n_events))
print("numeric {:.1f} seconds = {:.2f}ms/event".format(d_numeric, 1000. * d_numeric / n_events))
print("analytic {:.1f} seconds = {:.2f}ms/event".format(t_analytic, 1000. * t_analytic / n_events))
print("numeric {:.1f} seconds = {:.2f}ms/event".format(t_numeric, 1000. * t_numeric / n_events))
print("distance ", np.allclose(results_D, results_D_analytic, atol=1e-2, rtol=1e-10))
print("travel time ", np.allclose(results_T, results_T_analytic, atol=1e-2, rtol=1e-10))
print("asserting travel times")
testing.assert_allclose(results_T, results_T_analytic, atol=4e-2 * units.ns, rtol=1e-10)
print("asserting distances")
testing.assert_allclose(results_D, results_D_analytic, atol=3e-2 * units.m, rtol=1e-10)

print('T02test_analytic_D_T passed without issues')


from __future__ import absolute_import, division, print_function

def get_propagation_module(name='analytic'):
    """
    wrapper around all propagation modules
    
    The function returns the python class of the 
    respective propagation module
    
    Parameters
    ----------
    name: string
        * analytic: analytic ray tracer
    """
    if(name=='analytic'):
        from NuRadioMC.SignalProp.analyticraytracing import ray_tracing
        return ray_tracing
    else:
        raise NotImplementedError("module {} not implemented".format(name))



import pyPROPOSAL as pp
import numpy as np
from NuRadioMC.utilities import units
import os

"""
This module takes care of the PROPOSAL implementation. Some important things
should be considered.
Units: PROPOSAL used a fixed system of units that differs from that of NuRadioMC
and conversion between them must be done carefully. The definition of PROPOSAL
units can be found in this file. The most important are the energy unit (MeV)
and the distance unit (cm).
When a muon or a tau is propagated using PROPOSAL and its secondaries are obtained,
most of the secondaries belong to a DynamicData type. The ones we should care
about are the following:
- Brems: a bremsstrahlung photon
- DeltaE: an ionized electron
- EPair: an electron/positron pair
- Hadrons: a set of unespecified hadrons
- NuclInt: the products of a nuclear interaction
The last secondaries obtained via the propagation belong to the Particle DynamicData
type, and represent the products of the decay. They are standard particles with
a PDG code.
"""

# Units definition in PROPOSAL
pp_MeV = 1.e0
pp_GeV = 1.e3
pp_TeV = 1.e6
pp_PeV = 1.e9
pp_EeV = 1.e12
pp_ZeV = 1.e15

pp_m = 1.e2
pp_km = 1.e5

class secondary_properties:

    def __init__(self,
                 distance,
                 energy,
                 shower_type,
                 code,
                 name):
        self.distance = distance
        self.energy = energy
        self.shower_type = shower_type
        self.code = code
        self.name = name

    def __str__(self):
        s  = "Particle and code: {:} ({:})\n".format(self.name, self.code)
        s += "Energy in PeV: {:}\n".format(self.energy/units.PeV)
        s += "Distance from vertex in km: {:}\n".format(self.distance/units.km)
        s += "Shower type: {:}\n".format(self.shower_type)
        return s

def particle_code (particle):
    if particle.particle_def == pp.particle.GammaDef.get()      : return    0
    elif particle.particle_def == pp.particle.EMinusDef.get()   : return   11
    elif particle.particle_def == pp.particle.EPlusDef.get()    : return  -11
    elif particle.particle_def == pp.particle.NuEDef.get()      : return   12
    elif particle.particle_def == pp.particle.NuEBarDef.get()   : return  -12
    elif particle.particle_def == pp.particle.MuMinusDef.get()  : return   13
    elif particle.particle_def == pp.particle.MuPlusDef.get()   : return  -13
    elif particle.particle_def == pp.particle.NuMuDef.get()     : return   14
    elif particle.particle_def == pp.particle.NuMuBarDef.get()  : return  -14
    elif particle.particle_def == pp.particle.TauMinusDef.get() : return   15
    elif particle.particle_def == pp.particle.TauPlusDef.get()  : return  -15
    elif particle.particle_def == pp.particle.NuTauDef.get()    : return   16
    elif particle.particle_def == pp.particle.NuTauBarDef.get() : return  -16
    elif particle.particle_def == pp.particle.Pi0Def.get()      : return  111
    elif particle.particle_def == pp.particle.PiPlusDef.get()   : return  211
    elif particle.particle_def == pp.particle.PiMinusDef.get()  : return -211
    elif particle.particle_def == pp.particle.K0Def.get()       : return  130
    elif particle.particle_def == pp.particle.KPlusDef.get()    : return  310
    elif particle.particle_def == pp.particle.KMinusDef.get()   : return -310
    elif particle.particle_def == pp.particle.PPlusDef.get()    : return 2212
    elif particle.particle_def == pp.particle.PMinusDef.get()   : return-2212
    else: return None

def is_em_primary (particle):
    if particle.particle_def == pp.particle.EMinusDef.get(): return True
    elif particle.particle_def == pp.particle.EPlusDef.get(): return True
    elif particle.particle_def == pp.particle.GammaDef.get(): return True
    else:
       return False

def is_had_primary(particle):
    if particle.particle_def == pp.particle.PMinusDef.get(): return True
    elif particle.particle_def == pp.particle.PPlusDef.get(): return True
    elif particle.particle_def == pp.particle.Pi0Def.get(): return True
    elif particle.particle_def == pp.particle.PiMinusDef.get(): return True
    elif particle.particle_def == pp.particle.PiPlusDef.get(): return True
    elif particle.particle_def == pp.particle.K0Def.get(): return True
    elif particle.particle_def == pp.particle.KMinusDef.get(): return True
    elif particle.particle_def == pp.particle.KPlusDef.get(): return True
    else: return False

def is_shower_primary(particle):
    return is_em_primary(particle) or is_had_primary(particle)

datatype_code = {
    'Data.Brems'   : 81,
    'Data.DeltaE'  : 82,
    'Data.EPair'   : 83,
    'Data.Hadrons' : 84,
    'Data.NuclInt' : 85
}

em_datatypes = [
    'Data.Brems',
    'Data.DeltaE',
    'Data.EPair'
]

hadrons_datatypes = [
    'Data.Hadrons',
    'Data.NuclInt'
]

datatype_primaries = em_datatypes + hadrons_datatypes

particle_name = {
        0 : 'gamma',
       11 : 'e-',
      -11 : 'e+',
       12 : 'nu_e',
      -12 : 'nu_e_bar',
       13 : 'mu-',
      -13 : 'mu+',
       14 : 'nu_mu',
      -14 : 'nu_mu_bar',
       15 : 'tau-',
      -15 : 'tau+',
       16 : 'nu_tau',
      -16 : 'nu_tau_bar',
       81 : 'brems',
       82 : 'ionized_e',
       83 : 'e_pair',
       84 : 'hadrons',
       85 : 'nucl_int',
      111 : 'pi0',
      211 : 'pi+',
     -211 : 'pi-',
      130 : 'K0',
      310 : 'K+',
     -310 : 'K-',
     2212 : 'p+',
    -2212 : 'p-'
}

def filter_particle(secondaries, particle):
    prods = [p for p in secondaries if p.id == pp.particle.Data.Particle]
    E = [p.energy for p in prods if p.particle_def == particle]
    return sum(E)

def create_propagator(low=0.1*pp_PeV, particle_code=13, ecut=100*pp_TeV):
    mu_def_builder = pp.particle.ParticleDefBuilder()
    if (particle_code == 13):
        mu_def_builder.SetParticleDef(pp.particle.MuMinusDef.get())
    elif (particle_code == -13):
        mu_def_builder.SetParticleDef(pp.particle.MuPlusDef.get())
    elif (particle_code == 15):
        mu_def_builder.SetParticleDef(pp.particle.TauMinusDef.get())
    elif (particle_code == -15):
        mu_def_builder.SetParticleDef(pp.particle.TauPlusDef.get())
    else:
        error_str = "The propagation of this particle via PROPOSAL is not currently supported.\n"
        error_str += "Please choose between -/+muon (13/-13) and -/+tau (15/-15)"
        raise NotImplementedError(error_str)

    mu_def_builder.SetLow(low)
    mu_def = mu_def_builder.build()

    geometry = pp.geometry.Sphere(pp.Vector3D(), 1.e20, 0.0)
    medium = pp.medium.Ice(1.0)

    sec_def = pp.SectorDefinition()
    sec_def.medium = pp.medium.Ice(1.0)
    sec_def.geometry = geometry
    sec_def.particle_location = pp.ParticleLocation.inside_detector
    sec_def.cut_settings = pp.EnergyCutSettings(ecut, -1)
    sec_def.stopping_decay = True

    sec_def.scattering_model = pp.scattering.ScatteringModel.NoScattering
    sec_def.crosssection_defs.brems_def.lpm_effect = True
    sec_def.crosssection_defs.epair_def.lpm_effect = True
    # if another parametrization is wanted, it can be changed like this
    # sec_def.crosssection_defs.photo_def.parametrization = pp.parametrization.photonuclear.PhotoParametrization.BezrukovBugaev

    detector = geometry

    interpolation_def = pp.InterpolationDef()
    interpolation_def.path_to_tables = os.path.join(os.path.dirname(__file__), 'PROPOSAL_tables')
    interpolation_def.path_to_tables_readonly = os.path.join(os.path.dirname(__file__), 'PROPOSAL_tables')

    return pp.Propagator(mu_def, [sec_def], detector, interpolation_def)

def get_compact_sub_pev_losses(energy_arr, distance_arr, compact_dist, min_energy_loss):
    r""" return biggest compact loss if above min_energy_cut
    Parameters
    ----------
    energy_arr: array-like
        energy of the energy losses below min_energy_loss
    distance_arr: array_like
        distances of the energy losses below min_energy_loss
    compact_dist: float
        distance in centimeters (PROPOSAL units): how compact the energy losses should be
    min_energy_loss: float
        min energy for the sensitivity (here a PeV)
    """
    len_bins = np.arange(distance_arr[0], distance_arr[-1] + 1e-3, 100)
    # We have used 100 to create a bin length of 1 m
    len_indices = np.digitize(distance_arr, len_bins)
    bincount = np.bincount(len_indices, energy_arr)

    if len(bincount) <= compact_dist:
        sum_bins = np.sum(bincount)
        if sum_bins > min_energy_loss:
            return [sum_bins]
        else:
            return []

    # We transform the compact_dist into meters, since the above histogram
    # has a bin length of 1 m
    convolved_comp_arr = np.convolve(np.ones(int(compact_dist/pp_m)), bincount, mode='valid')
    if np.any(convolved_comp_arr > min_energy_loss):
        return [np.max(convolved_10m_arr)]
    else:
        return []

def produces_shower(particle, min_energy_loss=1*pp_PeV):

    energy_threshold = particle.energy > min_energy_loss
    if not energy_threshold:
        return False

    if particle.id == pp.particle.Data.Particle:
        shower_inducing = is_shower_primary(particle)
    elif str(particle.id) in datatype_primaries:
        shower_inducing = True
    else:
        return False

    return shower_inducing

def shower_properties(particle):

    if particle.id == pp.particle.Data.Particle:
        if is_em_primary(particle):
            shower_type = 'em'
        elif is_had_primary(particle):
            shower_type = 'had'
        else:
            return None, None, None

        code = particle_code(particle)

    elif str(particle.id) in em_datatypes:
        shower_type = 'em'
        code = datatype_code[str(particle.id)]

    elif str(particle.id) in hadrons_datatypes:
        shower_type = 'had'
        code = datatype_code[str(particle.id)]

    name = particle_name[code]

    return shower_type, code, name

def GetSecondaries(Elepton, lepton_code, random_seed=None):

    low = 0.1*pp_PeV # Low energy limit for the propagating particle
    propagation_length = 100*pp_km # Maximum propagation length
    compact_dist = 10*pp_m # Maximum distance for the compact losses

    if random_seed == None:
        random_seed = int( np.random.uniform(0,1e8) )

    pp.RandomGenerator.get().set_seed( random_seed )

    prop = create_propagator(low=low, particle_code=lepton_code)
    prop.particle.position = pp.Vector3D(0, 0, 0)
    prop.particle.direction = pp.Vector3D(0, 0, 1)
    prop.particle.propagated_distance = 0
    prop.particle.energy = Elepton / units.MeV # Proposal's energy unit is the MeV

    secondaries = prop.propagate(propagation_length)

    return secondaries

def GetSecondariesArray(Eleptons, lepton_codes, random_seed=None):

    low = 0.1*pp_PeV # Low energy limit for the propagating particle
    propagation_length = 100*pp_km # Maximum propagation length
    compact_dist = 10*pp_m # Maximum distance for the compact losses
    min_energy_loss = 1*pp_PeV # Minimal energy for a selected secondary-induced shower

    if random_seed == None:
        random_seed = int( np.random.uniform(0,1e8) )

    propagators = {}
    for lepton_code in np.unique(lepton_codes):
        if lepton_code not in propagators:
            propagators[lepton_code] = create_propagator(low=low, particle_code=lepton_code)

    secondaries_array = []

    for Elepton, lepton_code in zip(Eleptons, lepton_codes):

        propagators[lepton_code].particle.position = pp.Vector3D(0, 0, 0)
        propagators[lepton_code].particle.direction = pp.Vector3D(0, 0, 1)
        propagators[lepton_code].particle.propagated_distance = 0
        propagators[lepton_code].particle.energy = Elepton / units.MeV # Proposal's energy unit is the MeV

        secondaries = propagators[lepton_code].propagate(propagation_length)

        shower_inducing_prods = []

        for sec in secondaries:

            # Decays contain only one shower-inducing particle
            # Muons and neutrinos resulting from decays are ignored
            if produces_shower(sec, min_energy_loss):

                distance = sec.position.z * units.cm
                energy = sec.energy * units.MeV

                shower_type, code, name = shower_properties(sec)

                shower_inducing_prods.append( secondary_properties(distance, energy, shower_type, code, name) )

        secondaries_array.append(shower_inducing_prods)

    return secondaries_array

def GetDecays(Eleptons, lepton_codes, random_seed=None):

    low = 0.1*pp_PeV # Low energy limit for the propagating particle
    propagation_length = 100*pp_km # Maximum propagation length
    compact_dist = 10*pp_m # Maximum distance for the compact losses
    min_energy_loss = 1*pp_PeV # Minimal energy for a selected secondary-induced shower

    if random_seed == None:
        random_seed = int( np.random.uniform(0,1e8) )

    propagators = {}
    for lepton_code in lepton_codes:
        if lepton_code not in propagators:
            propagators[lepton_code] = create_propagator(low=low, particle_code=lepton_code)

    decays_array = []

    for Elepton, lepton_code in zip(Eleptons, lepton_codes):

        decay_prop = (None, None)

        while( decay_prop == (None,None) ):

            propagators[lepton_code].particle.position = pp.Vector3D(0, 0, 0)
            propagators[lepton_code].particle.direction = pp.Vector3D(0, 0, 1)
            propagators[lepton_code].particle.propagated_distance = 0
            propagators[lepton_code].particle.energy = Elepton / units.MeV # Proposal's energy unit is the MeV

            secondaries = propagators[lepton_code].propagate(propagation_length)

            decay_particles = np.array([p for p in secondaries if p.id == pp.particle.Data.Particle])
            decay_energies = np.array([p.energy for p in decay_particles])
            decay_energy = np.sum(decay_energies) * units.MeV

            try:
                decay_distance = decay_particles[0].position.z * units.cm
                decay_prop = (decay_distance, decay_energy)
                decays_array.append(decay_prop)
            except:
                decay_prop = (None, None)

    return np.array(decays_array)

def GetDecay(Elepton, lepton_code, random_seed=None):

    secondaries = GetSecondaries(Elepton, lepton_code)

    decay_particles = np.array([p for p in secondaries if p.id == pp.particle.Data.Particle])
    decay_energies = np.array([p.energy for p in decay_particles])
    decay_energy = np.sum(decay_energies) * units.MeV
    try:
        decay_distance = decay_particles[0].position.z * units.cm
    except:
        return (None, None)

    return (decay_distance, decay_energy)

def GetProdsArray(Eleptons, lepton_codes, aggregated_showers=False, random_seed=None):

    min_energy_loss = 1*pp_PeV # Minimal energy for a selected secondary-induced shower

    secondaries_array = GetSecondariesArray(Eleptons, lepton_codes, random_seed)

    shower_inducing_array = []

    for secondaries in secondaries_array:

        shower_inducing_prods = []

        if aggregated_showers:
            pass
        else:
            for sec in secondaries:
                # Decays contain only one shower-inducing particle
                # Muons and neutrinos resulting from decays are ignored
                if produces_shower(sec, min_energy_loss):

                    distance = sec.position.z * units.cm
                    energy = sec.energy * units.MeV

                    shower_type, code, name = shower_properties(sec)

                    shower_inducing_prods.append( secondary_properties(distance, energy, shower_type, code, name) )

        shower_inducing_array.append(shower_inducing_prods)

    return shower_inducing_array

def GetProds(Elepton, lepton_code, aggregated_showers=False, random_seed=None):

    min_energy_loss = 1*pp_PeV # Minimal energy for a selected secondary-induced shower

    secondaries = GetSecondaries(Elepton, lepton_code, random_seed)

    shower_inducing_prods = []

    if aggregated_showers:
        pass
    else:
        for sec in secondaries:
            # Decays contain only one shower-inducing particle
            # Muons and neutrinos resulting from decays are ignored
            if produces_shower(sec, min_energy_loss):

                distance = sec.position.z * units.cm
                energy = sec.energy * units.MeV

                shower_type, code, name = shower_properties(sec)

                shower_inducing_prods.append( secondary_properties(distance, energy, shower_type, code, name) )

    return shower_inducing_prods



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
import argparse
# import detector simulation modules
import NuRadioReco.modules.efieldToVoltageConverter
import NuRadioReco.modules.trigger.highLowThreshold
import NuRadioReco.modules.trigger.simpleThreshold
import NuRadioReco.modules.channelResampler
import NuRadioReco.modules.channelBandPassFilter
import NuRadioReco.modules.channelGenericNoiseAdder
from NuRadioReco.utilities import units
from NuRadioMC.simulation import simulation
import logging
import os
logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger("runstrawman")

# initialize detector sim modules
efieldToVoltageConverter = NuRadioReco.modules.efieldToVoltageConverter.efieldToVoltageConverter()
efieldToVoltageConverter.begin()
triggerSimulatorHighLow = NuRadioReco.modules.trigger.highLowThreshold.triggerSimulator()
triggerSimulatorSimple = NuRadioReco.modules.trigger.simpleThreshold.triggerSimulator()
channelResampler = NuRadioReco.modules.channelResampler.channelResampler()
channelBandPassFilter = NuRadioReco.modules.channelBandPassFilter.channelBandPassFilter()
channelGenericNoiseAdder = NuRadioReco.modules.channelGenericNoiseAdder.channelGenericNoiseAdder()

class mySimulation(simulation.simulation):


    def _detector_simulation(self):
        # start detector simulation
        if(bool(self._cfg['signal']['zerosignal'])):
            self._increase_signal(None, 0)

        efieldToVoltageConverter.run(self._evt, self._station, self._det)  # convolve efield with antenna pattern
        # downsample trace to internal simulation sampling rate (the efieldToVoltageConverter upsamples the trace to
        # 20 GHz by default to achive a good time resolution when the two signals from the two signal paths are added)
        channelResampler.run(self._evt, self._station, self._det, sampling_rate=1. / self._dt)

        if bool(self._cfg['noise']):
            Vrms = self._Vrms / (self._bandwidth /( 2.5 * units.GHz))** 0.5  # normalize noise level to the bandwidth its generated for
            channelGenericNoiseAdder.run(self._evt, self._station, self._det, amplitude=Vrms, min_freq=0 * units.MHz,
                                         max_freq=2.5 * units.GHz, type='rayleigh')

        # bandpass filter trace, the upper bound is higher then the sampling rate which makes it just a highpass filter
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[80 * units.MHz, 1000 * units.GHz],
                                  filter_type='butter', order=2)
        channelBandPassFilter.run(self._evt, self._station, self._det, passband=[0, 500 * units.MHz],
                                  filter_type='butter', order=10)

        # run a high/low trigger on the 4 downward pointing LPDAs
        triggerSimulatorHighLow.run(self._evt, self._station, self._det,
                                    threshold_high=2 * self._Vrms,
                                    threshold_low=-2 * self._Vrms,
                                    triggered_channels=None,  # select the LPDA channels
                                    number_concidences=1,  # 2/4 majority logic
                                    trigger_name='highlow_2sigma')

        # downsample trace back to detector sampling rate
#         channelResampler.run(self._evt, self._station, self._det, sampling_rate=self._sampling_rate_detector)


path = os.path.dirname(os.path.abspath(__file__))

parser = argparse.ArgumentParser(description='Run NuRadioMC simulation')
parser.add_argument('inputfilename', type=str,
                    help='path to NuRadioMC input event list')
parser.add_argument('detectordescription', type=str,
                    help='path to file containing the detector description')
parser.add_argument('config', type=str,
                    help='NuRadioMC yaml config file')
parser.add_argument('outputfilename', type=str,
                    help='hdf5 output filename')
parser.add_argument('outputfilenameNuRadioReco', type=str, nargs='?', default=None,
                    help='outputfilename of NuRadioReco detector sim file')
args = parser.parse_args()

sim = mySimulation(eventlist=os.path.join(path,args.inputfilename),
                            outputfilename=os.path.join(path,args.outputfilename),
                            detectorfile=os.path.join(path,args.detectordescription),
                            outputfilenameNuRadioReco=os.path.join(path,args.outputfilenameNuRadioReco),
                            config_file=os.path.join(path,args.config))
sim.run()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import numpy as np
from NuRadioReco.utilities import units
from scipy import interpolate as intp
from scipy import constants
from matplotlib import pyplot as plt
import os
import pickle
from time import time
from NuRadioMC.SignalGen.ARZ.ARZ import get_vector_potential_fast
import json

plot = True
outfile = 'interp_1e-3.json'

energy = 1.e6 * units.TeV
N = 1000
dt = 0.1 * units.ns
times = np.arange(0, N * dt, dt)
times = times + 0.5 * dt - times.mean()
#limits = ( int(N/2-25*units.ns/dt), int(N/2+25*units.ns/dt) )
n_index = 1.78
y = 0
ccnc = 'cc'
flavor = 12  # e = 12, mu = 14, tau = 16

cdir = os.path.dirname(__file__)
bins, depth_e, N_e = np.loadtxt(os.path.join(cdir, "../shower_library/nue_1EeV_CC_1_s0001.t1005"), unpack=True)
bins, depth_p, N_p = np.loadtxt(os.path.join(cdir, "../shower_library/nue_1EeV_CC_1_s0001.t1006"), unpack=True)
depth_e *= units.g / units.cm**2
depth_p *= units.g / units.cm**2
depth_e -= 1000 * units.g/units.cm**2  # all simulations have an artificial offset of 1000 g/cm^2
depth_p -= 1000 * units.g/units.cm**2
# sanity check if files electron and positron profiles are compatible
if (not np.all(depth_e == depth_p)):
    raise ImportError("electron and positron profile have different depths")

max_interp = 100
Rs = np.linspace(100, 1000, 10)
Rs = [100, 200, 500, 1000]
thetas = np.linspace(45, 65, 10)*units.deg
interp_factors_2 = np.linspace(1,100,100)
interp_factors_1 = np.linspace(1,100,100)
max_prod = 500
tol = 1.e-3
cut = 1.e-2

check_interp = False

res_dict = {}

for R in Rs:

    interp_results = {}

    for theta in thetas[:]:

        fields = []
        diffs = []

        vp_prec = get_vector_potential_fast(energy, theta, N, dt, depth_e, N_e-N_p, "EM", n_index, R, \
        interp_factor=max_interp, interp_factor2=max_interp, shift_for_xmax=True)
        field_prec = -np.array([np.diff(vp_prec[:,0]),np.diff(vp_prec[:,1]),np.diff(vp_prec[:,2])])/dt

        max_field_x = np.max(np.abs(field_prec[0]))
        max_field_z = np.max(np.abs(field_prec[2]))

        mean_field_x = np.max(np.abs(field_prec[0]))
        mean_field_z = np.max(np.abs(field_prec[2]))

        index_max_field_x = np.where(np.abs(field_prec[0]) == max_field_x)[0][0]
        index_max_field_z = np.where(np.abs(field_prec[2]) == max_field_z)[0][0]
        print(index_max_field_x, index_max_field_z)

        for interp_factor_1 in interp_factors_1:

            found = False

            tol1 = 0
            tol2 = 1

            for interp_factor_2 in interp_factors_2:

                if (interp_factor_1*interp_factor_2 > max_prod):
                    break

                prev_tol1 = tol1
                prev_tol2 = tol2

                print("R: ", R, ", theta: ", theta/units.deg, ", interpolation", interp_factor_1, interp_factor_2)

                vp = get_vector_potential_fast(energy, theta, N, dt, depth_e, N_e-N_p, "EM", n_index, R, \
                interp_factor=interp_factor_1, interp_factor2=interp_factor_2, shift_for_xmax=True)
                field = -np.array([np.diff(vp[:,0]),np.diff(vp[:,1]),np.diff(vp[:,2])])/dt
                #fields.append(field)

                diff = np.abs(field_prec-field)/np.abs(mean_field_x+mean_field_z)/2
                diff = np.nan_to_num(diff)
                #diffs.append(diff)

                #print(np.mean(diff[0]),np.mean(diff[2]))
                #plt.yscale('log')

                limits_x = [0,0]
                limits_x[0] = np.where( np.abs(field[0])[0:index_max_field_x] > cut*max_field_x )[0][0]
                limits_x[1] = np.where( np.abs(field[0])[index_max_field_x:] > cut*max_field_x )[0][-1] + index_max_field_x
                print("Time limits x: ", times[limits_x[0]], times[limits_x[1]])

                limits_z = [0,0]
                limits_z[0] = np.where( np.abs(field[2])[0:index_max_field_z] > cut*max_field_z )[0][0]
                limits_z[1] = np.where( np.abs(field[2])[index_max_field_z:] > cut*max_field_z )[0][-1] + index_max_field_z
                print("Time limits z: ", times[limits_z[0]], times[limits_z[1]])

                tol1 = np.sqrt(np.mean(diff[0][limits_x[0]:limits_x[1]]**2))
                tol2 = np.sqrt(np.mean(diff[2][limits_z[0]:limits_z[1]]**2))
                print(tol1, tol2)

                #if ( np.abs(prev_tol1-tol1) < tol/1e2 and np.abs(prev_tol2-tol2) < tol/1e2):
                     #and np.abs(tol1-tol) > 0.1*tol and np.abs(tol2-tol) > 0.1*tol ):
                    #break

                #if (np.mean(diff[0][diff[0]>0]) < tol and np.mean(diff[2][diff[2]>0]) < tol ):
                #if (np.min(diff[0][diff[0]>0]) < tol and np.min(diff[2][diff[2]>0]) < tol ):
                if ( (tol1 < tol and tol2 < tol) or check_interp == True or interp_factor_2 == 100 ):
                    interp_results[theta/units.deg] = (interp_factor_1, interp_factor_2)
                    found = True
                    print('found!')
                    if plot:
                        plt.plot(times,field_prec[0])
                        plt.plot(times[limits_x[0]:limits_x[1]],field[0][limits_x[0]:limits_x[1]])
                        plt.show()
                        plt.plot(times,field_prec[2])
                        plt.plot(times[limits_z[0]:limits_z[1]],field[2][limits_z[0]:limits_z[1]])
                        plt.show()
                        plt.plot(times, diff[0])
                        plt.yscale('log')
                        plt.show()
                    break
                plt.clf()

            if found:
                break
            else:
                print("Tolerance not achieved! Larger interp_factor_1 needed!")
    print("R: ", R)
    print("interp results", interp_results)
    res_dict[R] = dict(interp_results)

with open(outfile, 'w') as fout:
    json.dump(res_dict, fout, sort_keys=True, indent=4)
